/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import java.util.Hashtable;

import javax.swing.event.EventListenerList;

import com.onaro.client.leekui.jface.viewers.IPostSelectionProvider;
import com.onaro.client.leekui.jface.viewers.ISelection;
import com.onaro.client.leekui.jface.viewers.ISelectionChangedListener;
import com.onaro.client.leekui.jface.viewers.ISelectionProvider;
import com.onaro.client.leekui.jface.viewers.SelectionChangedEvent;
import com.onaro.client.leekui.ui.INullSelectionListener;
import com.onaro.client.leekui.ui.ISelectionListener;
import com.onaro.client.leekui.ui.ISelectionService;
import com.onaro.client.leekui.ui.IWorkbenchPart;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.AbstractSelectionService
 */
public abstract class AbstractSelectionService implements ISelectionService {

	/** 
     * The list of selection listeners (not per-part).
     */
    private EventListenerList listeners = new EventListenerList();

    /** 
     * The list of post selection listeners (not per-part).
     */
    private EventListenerList postListeners = new EventListenerList();

    /**
     * The currently active part.
     */
    IWorkbenchPart activePart;

    /**
     * The active part's selection provider, remembered in case the part 
     * replaces its selection provider after we hooked a listener.
     */
    private ISelectionProvider activeProvider;

    /**
     * Map from part id (String) to per-part tracker (AbstractPartSelectionTracker).
     */
    private Hashtable<String,Object> perPartTrackers;

    /**
     * The JFace selection listener to hook on the active part's selection provider.
     */
    private ISelectionChangedListener selListener = new ISelectionChangedListener() {
        public void selectionChanged(SelectionChangedEvent event) {
            fireSelection(activePart, event.getSelection());
        }
    };

    /**
     * The JFace post selection listener to hook on the active part's selection provider.
     */
    private ISelectionChangedListener postSelListener = new ISelectionChangedListener() {
        public void selectionChanged(SelectionChangedEvent event) {
            firePostSelection(activePart, event.getSelection());
        }
    };

    /**
     * Creates a new SelectionService.
     */
    protected AbstractSelectionService() {
    }

    /* (non-Javadoc)
     * Method declared on ISelectionService.
     */
    public void addSelectionListener(ISelectionListener l) {
        listeners.add(ISelectionListener.class, l);
    }

    /* (non-Javadoc)
     * Method declared on ISelectionService.
     */
    public void addSelectionListener(String partId, ISelectionListener listener) {
//        getPerPartTracker(partId).addSelectionListener(listener);
    	
    	throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * Method declared on ISelectionService.
     */
    public void addPostSelectionListener(ISelectionListener l) {
        postListeners.add(ISelectionListener.class, l);
    }

    /* (non-Javadoc)
     * Method declared on ISelectionService.
     */
    public void addPostSelectionListener(String partId,
            ISelectionListener listener) {
//        getPerPartTracker(partId).addPostSelectionListener(listener);
    	throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * Method declared on ISelectionService.
     */
    public void removeSelectionListener(ISelectionListener l) {
        listeners.remove(ISelectionListener.class, l);
    }

    /*
     * (non-Javadoc)
     * Method declared on ISelectionListener.
     */
    public void removePostSelectionListener(String partId,
            ISelectionListener listener) {
//        getPerPartTracker(partId).removePostSelectionListener(listener);
    	throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * Method declared on ISelectionService.
     */
    public void removePostSelectionListener(ISelectionListener l) {
        postListeners.remove(ISelectionListener.class, l);
    }

    /*
     * (non-Javadoc)
     * Method declared on ISelectionListener.
     */
    public void removeSelectionListener(String partId,
            ISelectionListener listener) {
//        getPerPartTracker(partId).removeSelectionListener(listener);
    	throw new UnsupportedOperationException();
    }

    /**
     * Fires a selection event to the given listeners.
     * 
     * @param part the part or <code>null</code> if no active part
     * @param sel the selection or <code>null</code> if no active selection
     */
    protected void fireSelection(final IWorkbenchPart part, final ISelection sel) {
        Object[] array = listeners.getListeners(ISelectionListener.class);
        for (int i = 0; i < array.length; i++) {
            final ISelectionListener l = (ISelectionListener) array[i];
            if ((part != null && sel != null)
                    || l instanceof INullSelectionListener) {
                
                try {
                    l.selectionChanged(part, sel);
                } catch (Exception e) {
                    WorkbenchPlugin.log(e);
                }
            }
        }
    }

    /**
     * Fires a selection event to the given listeners.
     * 
     * @param part the part or <code>null</code> if no active part
     * @param sel the selection or <code>null</code> if no active selection
     */
    protected void firePostSelection(final IWorkbenchPart part,
            final ISelection sel) {
        Object[] array = postListeners.getListeners(ISelectionListener.class);
        for (int i = 0; i < array.length; i++) {
            final ISelectionListener l = (ISelectionListener) array[i];
            if ((part != null && sel != null)
                    || l instanceof INullSelectionListener) {
                
                try {
                    l.selectionChanged(part, sel);
                } catch (Exception e) {
                    WorkbenchPlugin.log(e);
                }
            }
        }
    }

    /**
     * Returns the per-part selection tracker for the given part id.
     * 
     * @param partId part identifier
     * @return per-part selection tracker
     */
//    protected AbstractPartSelectionTracker getPerPartTracker(String partId) {
//        if (perPartTrackers == null) {
//            perPartTrackers = new Hashtable(4);
//        }
//        AbstractPartSelectionTracker tracker = (AbstractPartSelectionTracker) perPartTrackers
//                .get(partId);
//        if (tracker == null) {
//            tracker = createPartTracker(partId);
//            perPartTrackers.put(partId, tracker);
//        }
//        return tracker;
//    }

    /**
     * Creates a new per-part selection tracker for the given part id.
     * 
     * @param partId part identifier
     * @return per-part selection tracker
     */
//    protected abstract AbstractPartSelectionTracker createPartTracker(String partId);

    /**
     * Returns the selection.
     */
    public ISelection getSelection() {
        if (activeProvider != null) {
			return activeProvider.getSelection();
		} 
			return null;
		
    }

    /*
     * @see ISelectionService#getSelection(String)
     */
    public ISelection getSelection(String partId) {
//        return getPerPartTracker(partId).getSelection();
    	
    	throw new UnsupportedOperationException();
    }

    /**
     * Sets the current-active part (or null if none)
     * 
     * @since 3.1 
     *
     * @param newPart the new active part (or null if none)
     */
    public void setActivePart(IWorkbenchPart newPart) {
        // Optimize.
        if (newPart == activePart) {
			return;
		}
        ISelectionProvider selectionProvider = null;
        
        if (newPart != null) {
            selectionProvider = newPart.getSite().getSelectionProvider();
            
            if (selectionProvider == null) {
                newPart = null;
            }
        }
        
        if (newPart == activePart) {
			return;
		}
        
        if (activePart != null) {
            if (activeProvider != null) {
                activeProvider.removeSelectionChangedListener(selListener);
                if (activeProvider instanceof IPostSelectionProvider) {
					((IPostSelectionProvider) activeProvider)
                            .removePostSelectionChangedListener(postSelListener);
				} else {
					activeProvider
                            .removeSelectionChangedListener(postSelListener);
				}
                activeProvider = null;
            }
            activePart = null;
        }

        activePart = newPart;
        
        if (newPart != null) {
            activeProvider = selectionProvider;
            // Fire an event if there's an active provider
            activeProvider.addSelectionChangedListener(selListener);
            ISelection sel = activeProvider.getSelection();
            fireSelection(newPart, sel);
            if (activeProvider instanceof IPostSelectionProvider) {
				((IPostSelectionProvider) activeProvider)
                        .addPostSelectionChangedListener(postSelListener);
			} else {
				activeProvider.addSelectionChangedListener(postSelListener);
			}
            firePostSelection(newPart, sel);
        } else {
            fireSelection(null, null);
            firePostSelection(null, null);
        }
    }
}
