/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Window;
import java.util.concurrent.Callable;
import java.util.concurrent.Executors;

import javax.swing.SwingUtilities;

import com.jidesoft.swing.JideSwingUtilities;

/**
 * TODO: move this to another package.
 *
 * This class is a reimplementation of the Eclipse class org.eclipse.swt.custom.BusyIndicator
 */
public class BusyIndicator 
{
	public static void showWhile(Component parent, Runnable runnable)
	{
		Callable<Object> callable = Executors.callable(runnable);
		
		try 
		{
			showWhile(parent, callable);
		}
		catch (Exception e) {
			JideSwingUtilities.throwException(e);
		}
	}
	
	public static <T> T showWhile(Component parent, Callable<T> callable) throws Exception
	{
		if (callable == null) throw new IllegalArgumentException();
		
		Window windowAncestor = null;
		
		if (parent != null)
		{
			windowAncestor = SwingUtilities.getWindowAncestor(parent);
		}
		
		Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR); 
		
		Cursor origCursor = null;
		
		if (windowAncestor != null)
		{
			origCursor = windowAncestor.getCursor();
			
			windowAncestor.setCursor(waitCursor);
		}
		
		try
		{
		
			T result = callable.call();
			
			return result;
		}
		finally
		{
			if (windowAncestor != null)
			{
				windowAncestor.setCursor(origCursor);
			}
		}
	}
}
