/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.SafeRunner;
import org.osgi.framework.BundleContext;

import com.onaro.client.leekui.jface.util.SafeRunnable;
import com.onaro.client.leekui.ui.internal.registry.IWorkbenchRegistryConstants;
import com.onaro.client.leekui.ui.internal.registry.ViewDescriptor;

public class DashboardPlugin extends Plugin {

	static final Logger logger = Logger.getLogger(DashboardPlugin.class);

	// The plug-in ID
	public static final String PLUGIN_ID = "com.onaro.client.dashboard.ui"; //$NON-NLS-1$

	// The shared instance
	private static DashboardPlugin plugin;

	/**
	 * The constructor
	 */
	public DashboardPlugin() {
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.Plugins#start(org.osgi.framework.BundleContext)
	 */
	@Override
	public void start(BundleContext context) throws Exception {

		super.start(context);

		processExtensions();
	}

	private void processExtensions() {
		try {
			processExtensionsImpl();
		} catch (Exception e) {
			logger.warn(e.getMessage(), e);
		}
	}

	private void processExtensionsImpl() throws Exception {
		IExtensionRegistry extensionRegistry = Platform.getExtensionRegistry();

		IExtensionPoint viewsExtensionPoint = extensionRegistry.getExtensionPoint(PLUGIN_ID + ".views"); //$NON-NLS-1$

		if (viewsExtensionPoint != null) {
			IConfigurationElement[] configurationElements = viewsExtensionPoint.getConfigurationElements();

			for (final IConfigurationElement configurationElement : configurationElements) {

				SafeRunnable runnable = new SafeRunnable() {

					public void run() throws Exception {

						String configurationElementName = configurationElement.getName();

						if (IWorkbenchRegistryConstants.TAG_VIEW.equals(configurationElementName)) {
							ViewDescriptor viewDescriptor = new ViewDescriptor(configurationElement);
							Workbench.getInstance().getViewRegistry().add(viewDescriptor);
						}
					}

				};

				SafeRunner.run(runnable);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.Plugin#stop(org.osgi.framework.BundleContext)
	 */
	@Override
	public void stop(BundleContext context) throws Exception {
		plugin = null;

		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static DashboardPlugin getDefault() {
		return plugin;
	}

}
