/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.SwingUtilities;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.jidesoft.document.DocumentComponent;
import com.jidesoft.document.DocumentPane;
import com.onaro.client.leekui.ui.IEditorDescriptor;
import com.onaro.client.leekui.ui.IEditorInput;
import com.onaro.client.leekui.ui.IEditorPart;
import com.onaro.client.leekui.ui.IEditorReference;
import com.onaro.client.leekui.ui.IEditorRegistry;
import com.onaro.client.leekui.ui.IMemento;
import com.onaro.client.leekui.ui.IWorkbenchPage;
import com.onaro.client.leekui.ui.PartInitException;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.EditorManager
 */
public class EditorManager {
	final WorkbenchPage page;

	public EditorManager(WorkbenchPage page) {
		if (page == null) throw new IllegalArgumentException();

		this.page = page;
	}

	public IEditorPart findEditor(String editorId, IEditorInput input, int matchFlags) {
		List<IEditorReference> refs = findEditors(input, editorId, matchFlags);
		if (refs.isEmpty()) {
			return null;
		}
		return refs.get(0).getEditor(true);
	}

	public List<IEditorReference> findEditors(IEditorInput input, String editorId, int matchFlags) {
		if (matchFlags == IWorkbenchPage.MATCH_NONE) {
			return Collections.emptyList();
		}

		List<IEditorReference> matchedEditors = new ArrayList<IEditorReference>();

		List<IEditorReference> allEditors = page.getEditorReferences();

		if (!allEditors.isEmpty()) {
			// TODO: write this in v4.0

			throw new UnsupportedOperationException();
		}

		return matchedEditors;
	}

	public IEditorRegistry getEditorRegistry() {
		return Workbench.getInstance().getEditorRegistry();
	}

	public IEditorReference openEditor(String editorId, IEditorInput input, boolean setVisible)
			throws PartInitException {
		if (editorId == null || input == null) {
			throw new IllegalArgumentException();
		}

		IEditorRegistry reg = getEditorRegistry();

		IEditorDescriptor desc = reg.findEditor(editorId);

		if (desc == null) {
			String msg = null;
			throw new PartInitException(msg);
		}

		IEditorReference result = openEditorFromDescriptor(desc, input);

		return result;
	}

	private IEditorReference openEditorFromDescriptor(IEditorDescriptor desc, IEditorInput input)
			throws PartInitException {
		EditorReference editor = null;

		editor = reuseInternalEditor(desc, input);
		if (editor == null) {
			editor = new EditorReference(this, input, desc);
		}

		final DocumentPane documentPane = page.getDocumentPane();

		final EditorPane editorPane = (EditorPane) editor.getPane();

		final DocumentComponent docComp = WorkbenchPageUtils.createDocumentComponent(editorPane);

		Runnable openDocRunnable = new Runnable() {
			public void run() {
				documentPane.openDocument(docComp);
			}
		};

		SwingUtilities.invokeLater(openDocRunnable);

		Runnable runnable = new Runnable()
		{
			public  void run()
			{
				// force opening of editor
				editorPane.getEditorReference().getEditor(true);

			}
		};
		
		//TODO: do this in the correct threading way
		Thread t = new Thread(runnable);
		t.start();
		
		// Workbench wb = (Workbench) window.getWorkbench();
		// wb.getEditorHistory().add(input, desc);
		return editor;
	}

	private EditorReference reuseInternalEditor(IEditorDescriptor desc, IEditorInput input) {
		// TODO write this in v4.0

		return null;
	}

	public IStatus saveState(IMemento childMem) {
		// TODO write this in v4.0
		
		return Status.OK_STATUS;
	}

}
