/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;

import com.onaro.client.leekui.ui.IEditorDescriptor;
import com.onaro.client.leekui.ui.IEditorInput;
import com.onaro.client.leekui.ui.IEditorPart;
import com.onaro.client.leekui.ui.IEditorReference;
import com.onaro.client.leekui.ui.PartInitException;
import com.onaro.client.leekui.ui.internal.compatibility.EditorDescriptor;
import com.onaro.client.leekui.ui.internal.part.NullEditorInput;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.EditorReference
 */
public final class EditorReference extends WorkbenchPartReference implements IEditorReference {

	IEditorInput restoredInput;

	EditorManager manager;

	public EditorReference(EditorManager manager, IEditorInput input, IEditorDescriptor desc) {
		this.manager = manager;
		restoredInput = input;
		init(desc.getId(), desc.getLabel(), "", desc.getIcon(), desc.getLabel(), ""); //$NON-NLS-1$ //$NON-NLS-2$
	}

	public IEditorPart getEditor(boolean restore) {
		return (IEditorPart) getPart(restore);
	}

	@Override
	protected IEditorPart createPart() {
		IEditorPart part = null;

		try {
			part = createPartHelper();
		}
		catch (Throwable t) {
			WorkbenchPlugin.log(t);
		}

		if (part == null) {
			// TODO: create an error part

			return null;
		}

		return part;
	}

	public IEditorInput getEditorInput() throws PartInitException {
		if (isDisposed()) {
			if (!(restoredInput instanceof NullEditorInput)) {
				restoredInput = new NullEditorInput();
			}
			return restoredInput;
		}

		IEditorPart part = getEditor(false);
		if (part != null) {
			return part.getEditorInput();
		}

		return getRestoredInput();
	}

	private IEditorInput getRestoredInput() throws PartInitException {
		if (restoredInput != null) {
			return restoredInput;
		}

		// TODO: finish this in v4.0

		return null;
	}

	private IEditorPart createPartHelper() throws PartInitException {
		EditorDescriptor descriptor = getDescriptor();

		final IEditorPart editor;

		IEditorInput editorInput = getEditorInput();

		try {
			editor = descriptor.createEditor();
		}
		catch (Throwable t) {
			String msg = null;
			throw new PartInitException(msg, t);
		}

		final EditorPane editorPane = (EditorPane) getPane();

		EditorSite editorSite = new EditorSite(this, editor, manager.page, descriptor);

		editor.init(editorSite, editorInput);

		final JComponent editorPartComponent = editor.createPartComponent();

		Runnable runnable = new Runnable() {
			public void run() {
				editorPane.setEditorComponent(editorPartComponent);
			}
		};

		SwingUtilities.invokeLater(runnable);

		return editor;
	}

	public EditorDescriptor getDescriptor() {
		EditorDescriptor descriptor = getDescriptor(getId());

		return descriptor;
	}

	private EditorDescriptor getDescriptor(String id) {
		return (EditorDescriptor) Workbench.getInstance().getEditorRegistry().findEditor(id);
	}

	@Override
	protected PartPane createPane() {
		return new EditorPane(this, manager.page);
	}
}
