/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import com.jidesoft.docking.DefaultDockingManager;
import com.jidesoft.docking.DockContext;
import com.jidesoft.docking.DockableFrame;
import com.jidesoft.document.DocumentPane;
import com.onaro.client.leekui.ui.IPageLayout;
import com.onaro.client.leekui.ui.IPerspectiveDescriptor;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.PageLayout
 */
public class PageLayout implements IPageLayout {

	private final Perspective perspective;

	public PageLayout(Perspective perspective) {
		if (perspective == null) throw new IllegalArgumentException();

		this.perspective = perspective;
	}

	protected WorkbenchPage getPage() {
		return perspective.page;
	}

	protected DefaultDockingManager getDockingManager() {
		return perspective.getDockingManager();
	}

	protected DocumentPane getDocumentPane() {
		return getPage().getDocumentPane();
	}

	public static boolean isFixed(WorkbenchPage page) {
		// TODO: write this in v4.0

		return false;
	}

	public boolean isFixed() {

		return isFixed(getPage());
	}

	public void setFixed(boolean fixed) {
		// TODO: write this in v4.0
	}
	
	private void addFrame(DockableFrame frame)
	{
		DefaultDockingManager dockingManager = getDockingManager();
		
		if (dockingManager.getFrame(frame.getKey()) != null) return;
		
		dockingManager.addFrame(frame);
	}

	public void addView(String viewId, Relationship relationship, String refId) {

		WorkbenchPage page = getPage();

		DefaultDockingManager dockingManager = getDockingManager();

		try {

			ViewPane viewPane = WorkbenchPageUtils.createViewPane(page, viewId, null);

			DockableFrame dockableFrame = perspective.getPresentation().getDockableFrame(viewPane, true);

			if (IPageLayout.ID_EDITOR_AREA.equals(refId) || refId == null) {

				int dockContextSide = getDockContextSide(relationship);

				dockableFrame.setInitSide(dockContextSide);

				addFrame(dockableFrame);
			}
			else {

				String refDockableFrameKey = WorkbenchPageUtils.getDockableFrameKey(refId);

				if (refDockableFrameKey == null) {
					String msg = null;
					throw new RuntimeException(msg);
				}

				addFrame(dockableFrame);

				String dockableFrameKey = dockableFrame.getKey();

				if (relationship == null) {
					dockingManager.moveFrame(dockableFrameKey, refDockableFrameKey);
				}
				else {
					int side = getDockContextSide(relationship);

					dockingManager.moveFrame(dockableFrameKey, refDockableFrameKey, side);
				}

			}
		}
		catch (Throwable t) {
			WorkbenchPlugin.log(t);
		}
	}

	private int getDockContextSide(Relationship relationship) {
		switch (relationship) {
			case TOP:
				return DockContext.DOCK_SIDE_NORTH;
			case BOTTOM:
				return DockContext.DOCK_SIDE_SOUTH;
			case LEFT:
				return DockContext.DOCK_SIDE_WEST;
			case RIGHT:
				return DockContext.DOCK_SIDE_EAST;
			default:
				throw new IllegalArgumentException();
		}
	}

	public IPerspectiveDescriptor getDescriptor() {
		return perspective.getDescriptor();
	}

	public String getEditorArea() {
		return ID_EDITOR_AREA;
	}

	public boolean isEditorAreaVisible() {
		return getDockingManager().isShowWorkspace();
	}

	public void setEditorAreaVisible(boolean showEditorArea) {
		getDockingManager().setShowWorkspace(showEditorArea);
	}
}
