/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import com.onaro.client.leekui.jface.viewers.ISelectionProvider;
import com.onaro.client.leekui.ui.IActionBars;
import com.onaro.client.leekui.ui.IWorkbenchPage;
import com.onaro.client.leekui.ui.IWorkbenchPart;
import com.onaro.client.leekui.ui.IWorkbenchPartReference;
import com.onaro.client.leekui.ui.IWorkbenchPartSite;
import com.onaro.client.leekui.ui.IWorkbenchWindow;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.PartPane
 */
public abstract class PartSite implements IWorkbenchPartSite {
	private IWorkbenchPartReference partReference;

	private IWorkbenchPart part;

	private IWorkbenchPage page;

	private String extensionID;

	private IActionBars actionBars;

	private ISelectionProvider selectionProvider;

	public PartSite(IWorkbenchPartReference ref, IWorkbenchPart part, IWorkbenchPage page) {
		this.partReference = ref;
		this.part = part;
		this.page = page;

		// because of obfuscation, there is no package
		// extensionID = IWorkbench.class.getPackage().getName() + ".UnknownID";
		extensionID = "com.onaro.client.leekui.ui" + ".UnknownID"; //$NON-NLS-1$ //$NON-NLS-2$

	}

	public IWorkbenchPartReference getPartReference() {
		return partReference;
	}

	public IWorkbenchPage getPage() {
		return page;
	}

	public IWorkbenchPart getPart() {
		return part;
	}

	public String getId() {
		return extensionID;
	}

	/**
	 * Sets the part registry extension ID.
	 * 
	 * @param id
	 *            the registry extension ID
	 */
	protected void setId(String id) {
		extensionID = id;
	}

	public IWorkbenchWindow getWorkbenchWindow() {
		return page.getWorkbenchWindow();
	}

	/**
	 * Sets the part.
	 */
	public void setPart(IWorkbenchPart newPart) {
		part = newPart;
	}

	public IActionBars getActionBars() {

		return actionBars;
	}

	public void dispose() {

		actionBars = null;
	}

	public void setActionBars(IActionBars actionBars) {
		this.actionBars = actionBars;
	}

	public ISelectionProvider getSelectionProvider() {
		return selectionProvider;
	}

	public void setSelectionProvider(ISelectionProvider selectionProvider) {
		this.selectionProvider = selectionProvider;
	}

}
