/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import java.awt.Frame;
import java.awt.KeyboardFocusManager;
import java.awt.Window;
import java.lang.reflect.InvocationTargetException;

import javax.swing.FocusManager;
import javax.swing.SwingUtilities;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.util.Util
 */
public class Util 
{
	/**
	 * Ensures that a string is not null. Converts null strings into empty strings, and leaves any other string
	 * unmodified. Use this to help wrap calls to methods that return null instead of the empty string. Can also help
	 * protect against implementation errors in methods that are not supposed to return null.
	 * 
	 * @param input
	 *            input string (may be null)
	 * @return input if not null, or the empty string if input is null
	 */
	public static String safeString(String input) {
		if (input != null) {
			return input;
		}

		return ""; //$NON-NLS-1$
	}
    
    /**
     * Invoke the Runnable in the AWT event dispatching thread.
     * 
     * Unlike the original invokeAndWait method, this one CAN be executed from the Swing / Event Dispatching thread.
     * 
     * @throws InterruptedException
     * @throws InvocationTargetException
     * 
     * @see SwingUtilities#invokeAndWait(Runnable)
     */
    public static void invoke(Runnable runnable) throws InterruptedException, InvocationTargetException {

        if (runnable == null) throw new IllegalArgumentException("callable"); //$NON-NLS-1$

        if (SwingUtilities.isEventDispatchThread()) {
            runnable.run();
        } else {
            SwingUtilities.invokeAndWait(runnable);
        }
    }
    
    /**
     * Does not throw exceptions.
     * 
     * @see OnaroSwingUtilities#invoke(Runnable)
     */
    public static void invokeSafely(Runnable runnable) {

        try {
            invoke(runnable);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public static Window getActiveWindow() {
        KeyboardFocusManager currentKeyboardFocusManager = FocusManager.getCurrentKeyboardFocusManager();
        if (currentKeyboardFocusManager != null) {
            Window activeWindow = currentKeyboardFocusManager.getActiveWindow();
            if (activeWindow != null) {
                return activeWindow;
            }
        }
        
        Frame[] frames = Frame.getFrames();
        for (Frame frame : frames) {
            if (frame.isActive()) return frame;
        }
        
        if (frames.length > 0) {
            return frames[0];
        }
        
        return null;
    }
}
