/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import java.awt.Insets;

import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.ButtonModel;
import javax.swing.JButton;
import javax.swing.JToggleButton;
import javax.swing.JToggleButton.ToggleButtonModel;

import com.onaro.client.leekui.jface.action.IStatusLineManager;
import com.onaro.client.leekui.jface.action.IToolBarManager;
import com.onaro.client.leekui.ui.IActionBars;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.ViewActionBars
 */
public class ViewActionBars implements IActionBars {

	final ViewReference viewReference;
	
	private ToolBarManager toolBarManager;
	
	private StatusLineManager statusLineManager;
	
	public ViewActionBars(ViewReference viewReference)
	{
		if (viewReference == null) throw new IllegalArgumentException();
		
		this.viewReference = viewReference;
	}
	
	public StatusLineManager getStatusLineManager()
	{
		if (statusLineManager == null)
		{
			statusLineManager = new StatusLineManager();
		}
		
		return statusLineManager;
	}
	
	public ToolBarManager getToolBarManager()
	{
		if (toolBarManager == null)
		{
			toolBarManager = new ToolBarManager();
		}
		
		return toolBarManager;
	}
	
	class StatusLineManager implements IStatusLineManager
	{
		
	}

	class ToolBarManager implements IToolBarManager
	{

		public ButtonModel addButton(Action action) {
			JButton button = new JButton(action);
			
			return add(button);
		}

		public ToggleButtonModel addToggleButton(Action action) {
			
			JToggleButton button = new JToggleButton(action);
			
			return (ToggleButtonModel) add(button);
		}
		
		private ButtonModel add(AbstractButton button)
		{
			button.setMargin(new Insets(2, 2, 2, 2));
			button.setText(""); //$NON-NLS-1$
//			button.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
			button.setFocusable(false);
//			button.setBackground(null);
//			button.setOpaque(true);

			ViewPane viewPane = (ViewPane) viewReference.getPane();
			
			viewPane.getToolBar().add(button);
			
			return button.getModel();
		}
	}
}
