/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import com.onaro.client.leekui.ui.IViewReference;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import java.awt.BorderLayout;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.ViewPane
 */
public final class ViewPane extends PartPane {
	private static final long serialVersionUID = 1L;

	/**
	 * The real toolBar.
	 */
	private JToolBar toolBar;
	
	/**
	 * A filler toolBar, only used for spacing.
	 */
	private JToolBar fillerToolBar;
	
	private JPanel northPanel;

	public ViewPane(IViewReference viewRef, WorkbenchPage page) {
		super(viewRef, page);

		initialize();
	}

	private void initialize() {
//		this.setBorder(BorderFactory.createEtchedBorder());
		this.setLayout(new BorderLayout());
//		this.setMinimumSize(new Dimension(200, 200));

		add(getNorthPanel(), BorderLayout.NORTH);
	}
	
	private JPanel getNorthPanel()
	{
		if (northPanel == null)
		{
			northPanel = new JPanel(new BorderLayout());
			
			northPanel.add(getFillerToolBar(), BorderLayout.CENTER);
			northPanel.add(getToolBar(), BorderLayout.EAST);
		}
		
		return northPanel;
	}
	
	private JToolBar getFillerToolBar()
	{
		if (fillerToolBar == null)
		{
			fillerToolBar = new JToolBar();
			initialize(fillerToolBar);
		}
		
		return fillerToolBar;
	}

	public JToolBar getToolBar() {
		if (toolBar == null) {

			toolBar = new JToolBar();
			initialize(toolBar);
		}

		return toolBar;
	}
	
	private void initialize(JToolBar toolBar)
	{
		toolBar.setFloatable(false);
		toolBar.setRollover(true);
	}

	public ViewReference getViewReference() {
		return (ViewReference) getPartReference();
	}

	public void setViewComponent(JComponent component) {
		add(component, BorderLayout.CENTER);
	}
}
