/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import static com.onaro.client.leekui.ui.internal.WorkbenchPageUtils.getDockableFrameKey;
import static com.onaro.client.leekui.ui.internal.WorkbenchPageUtils.getEditorReference;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.KeyboardFocusManager;
import java.awt.Window;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.Callable;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.RootPaneContainer;
import javax.swing.SwingUtilities;
import javax.swing.border.LineBorder;
import javax.swing.plaf.UIResource;

import org.apache.commons.lang.ObjectUtils;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.osgi.util.NLS;

import com.jidesoft.docking.DefaultDockingManager;
import com.jidesoft.docking.DockingManager;
import com.jidesoft.document.DocumentComponent;
import com.jidesoft.document.DocumentPane;
import com.jidesoft.swing.JideTabbedPane;
import com.onaro.client.leekui.jface.viewers.ISelection;
import com.onaro.client.leekui.ui.IEditorInput;
import com.onaro.client.leekui.ui.IEditorPart;
import com.onaro.client.leekui.ui.IEditorReference;
import com.onaro.client.leekui.ui.IMemento;
import com.onaro.client.leekui.ui.IPartListener;
import com.onaro.client.leekui.ui.IPerspectiveDescriptor;
import com.onaro.client.leekui.ui.ISelectionListener;
import com.onaro.client.leekui.ui.IViewPart;
import com.onaro.client.leekui.ui.IViewReference;
import com.onaro.client.leekui.ui.IWorkbenchPage;
import com.onaro.client.leekui.ui.IWorkbenchPart;
import com.onaro.client.leekui.ui.IWorkbenchPartReference;
import com.onaro.client.leekui.ui.IWorkbenchPartSite;
import com.onaro.client.leekui.ui.IWorkbenchWindow;
import com.onaro.client.leekui.ui.MultiEditor;
import com.onaro.client.leekui.ui.PartInitException;
import com.onaro.client.leekui.ui.PlatformUI;
import com.onaro.client.leekui.ui.WorkbenchException;
import com.onaro.client.leekui.ui.internal.compatibility.PerspectiveDescriptor;
import com.onaro.client.leekui.ui.internal.registry.PerspectiveRegistry;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.WorkbenchPage
 */
public class WorkbenchPage implements IWorkbenchPage {

	static {
		KeyboardFocusManager keyboardFocusManager = KeyboardFocusManager.getCurrentKeyboardFocusManager();

		keyboardFocusManager.addPropertyChangeListener("focusOwner", new KeyboardFocusManagerPropertyChangeListener()); //$NON-NLS-1$
	}

	private static final long serialVersionUID = 1L;

	private ViewFactory viewFactory;

	private EditorManager editorManager;

	private PerspectiveList perspList;

	private WorkbenchPageComponent component;

	PageSelectionService selectionService;

	private IPerspectiveDescriptor deferredActivePerspective;

	IWorkbenchPartReference activePartRef;

	public WorkbenchPage() {

		perspList = new PerspectiveList();

		selectionService = new PageSelectionService(this);
	}

	public final WorkbenchPageComponent getComponent() {

		if (component == null) {
			component = new WorkbenchPageComponent();
		}

		return component;
	}

	public void activate(IWorkbenchPart part) {

		if (part instanceof MultiEditor) {
			part = ((MultiEditor) part).getActiveEditor();
		}

		if (part instanceof IEditorPart) {
			activate((IEditorPart) part);
		}
		else if (part instanceof IViewPart) {
			activate((IViewPart) part);
		}
	}

	public void activate(IEditorPart editorPart) {
		// TODO: write this in v4.0
	}

	public void bringToTop(IWorkbenchPart part) {

		if (part instanceof MultiEditor) {
			part = ((MultiEditor) part).getActiveEditor();
		}

		if (part instanceof IEditorPart) {
			bringToTop((IEditorPart) part);
		}
		else if (part instanceof IViewPart) {
			bringToTop((IViewPart) part);
		}
	}

	public void bringToTop(IEditorPart editorPart) {
		// TODO: write this in v4.0
	}

	public void bringToTop(IViewPart viewPart) {
		String key = getDockableFrameKey(viewPart);

		if (key == null) return;

		getActivePerspective().getDockingManager().showFrame(key, false);
	}

	public void activate(IViewPart viewPart) {
		String key = getDockableFrameKey(viewPart);

		if (key == null) return;

		try {
			getActivePerspective().getDockingManager().showFrame(key, true);
		}
		catch (Exception e) {
			// to catch a bug reported by ofer
			WorkbenchPlugin.log(e);
		}
	}

	public EditorManager getEditorManager() {
		if (editorManager == null) {
			editorManager = new EditorManager(this);
		}

		return editorManager;
	}

	public ViewFactory getViewFactory() {
		if (viewFactory == null) {
			viewFactory = new ViewFactory(this, Workbench.getInstance().getViewRegistry());
		}
		return viewFactory;
	}

	public IEditorPart openEditor(IEditorInput input, String editorId) throws PartInitException {

		return openEditor(input, editorId, true);

	}

	public IEditorPart openEditor(IEditorInput input, String editorId, boolean activate) throws PartInitException {
		return openEditor(input, editorId, activate, MATCH_INPUT);
	}

	public IEditorPart openEditor(final IEditorInput input, final String editorId, final boolean activate,
			final int matchFlags) throws PartInitException {
		Callable<IEditorPart> callable = new Callable<IEditorPart>() {
			public IEditorPart call() throws PartInitException {
				return busyOpenEditor(input, editorId, activate, matchFlags);
			}
		};

		try {
			return BusyIndicator.showWhile(getComponent(), callable);
		}
		catch (PartInitException pie) {
			throw pie;
		}
		catch (Exception e) {
			throw new PartInitException(e);
		}
	}

	protected IEditorPart busyOpenEditor(IEditorInput input, String editorId, boolean activate, int matchFlags)
			throws PartInitException {

		IEditorPart editor = getEditorManager().findEditor(editorId, input, matchFlags);

		// Otherwise, create a new one. This may cause the new editor to
		// become the visible (i.e top) editor.
		IEditorReference ref = null;
		ref = getEditorManager().openEditor(editorId, input, true);

		if (ref != null) {
			editor = ref.getEditor(true);
		}

		if (editor != null) {

			if (activate) {
				if (editor instanceof MultiEditor) {
					activate(((MultiEditor) editor).getActiveEditor());
				}
				else {
					activate(editor);
				}
			}
			else {
				bringToTop(editor);
			}

			// window.firePerspectiveChanged(this, getPerspective(), ref,
			// CHANGE_EDITOR_OPEN);
			// window.firePerspectiveChanged(this, getPerspective(),
			// CHANGE_EDITOR_OPEN);
		}

		return editor;
	}

	public IViewPart showView(String viewId) throws PartInitException {
		return showView(viewId, null, ViewMode.ACTIVATE);
	}

	public IViewPart showView(final String viewId, final String secondaryId, final ViewMode viewMode)
			throws PartInitException {

		Callable<IViewPart> callable = new Callable<IViewPart>() {
			public IViewPart call() throws PartInitException {
				return busyShowView(viewId, secondaryId, viewMode);
			}
		};

		try {
			return BusyIndicator.showWhile(getComponent(), callable);
		}
		catch (PartInitException pie) {
			throw pie;
		}
		catch (Exception e) {
			throw new PartInitException(e);
		}
	}

	public List<IEditorReference> getEditorReferences() {
		List<IEditorReference> editorReferences = new ArrayList<IEditorReference>();

		DocumentPane documentPane = getDocumentPane();

		for (String documentName : documentPane.getDocumentNames()) {
			DocumentComponent docComponent = documentPane.getDocument(documentName);

			IEditorReference editorReference = getEditorReference(docComponent);

			if (editorReference != null) {
				editorReferences.add(editorReference);
			}
		}

		return editorReferences;
	}

	public Perspective getActivePerspective() {
		return perspList.getActive();
	}

	protected IViewPart busyShowView(String viewId, String secondaryId, ViewMode mode) throws PartInitException {

		Perspective persp = getActivePerspective();
		if (persp == null) {
			return null;
		}

		// If this view is already visible just return.
		IViewReference ref = persp.findView(viewId, secondaryId);
		IViewPart view = null;
		if (ref != null) {
			view = ref.getView(true);
		}
		if (view != null) {
			busyShowView(view, mode);
			return view;
		}

		// Show the view.
		view = persp.showView(viewId, secondaryId);
		if (view != null) {
			busyShowView(view, mode);

			// IWorkbenchPartReference partReference = getReference(view);

			// window.firePerspectiveChanged(this, getPerspective(),
			// partReference, CHANGE_VIEW_SHOW);
			// window.firePerspectiveChanged(this, getPerspective(),
			// CHANGE_VIEW_SHOW);
		}

		return view;
	}

	public IWorkbenchPartReference getReference(IWorkbenchPart part) {
		if (part == null) {
			return null;
		}
		IWorkbenchPartSite site = part.getSite();
		if (!(site instanceof PartSite)) {
			return null;
		}
		PartSite partSite = ((PartSite) site);

		// PartPane pane = partSite.getPane();
		// if (pane instanceof MultiEditorInnerPane) {
		// MultiEditorInnerPane innerPane = (MultiEditorInnerPane) pane;
		// return innerPane.getParentPane().getPartReference();
		// }

		return partSite.getPartReference();
	}

	private void busyShowView(IViewPart part, ViewMode viewMode) {

		switch (viewMode) {
			case ACTIVATE: {
				activate(part);
				break;
			}
			case VISIBLE: {
				bringToTop(part);
				break;
			}
			default:
				// do nothing
		}
	}

	public DocumentPane getDocumentPane() {
		return getComponent().getDocumentPane();
	}

	public PerspectiveHelper getPerspectiveHelper() {
		return getActivePerspective().getPresentation();
	}

	public void setPerspective(final IPerspectiveDescriptor desc) {

		BusyIndicator.showWhile(null, new Runnable() {
			public void run() {
				busySetPerspective(desc);
			}
		});
	}

	void busySetPerspective(IPerspectiveDescriptor perspectiveDescriptor) {

		Perspective newPersp = null;
		newPersp = findPerspective(perspectiveDescriptor);
		if (newPersp == null) {
			PerspectiveDescriptor realDesc = (PerspectiveDescriptor) perspectiveDescriptor;
			newPersp = createPerspective(realDesc, true);
			if (newPersp == null) {
				return;
			}
		}

		// Change layout.
		setPerspective(newPersp);

	}

	private void setPerspective(Perspective newPerspective) {

		final Perspective oldPerspective = getActivePerspective();

		if (ObjectUtils.equals(newPerspective, oldPerspective)) return;

		try {
			if (oldPerspective != null) {
				// fire the pre-deactivate
				// window.firePerspectivePreDeactivate(this, oldPersp.getDesc());
			}

			if (newPerspective != null) {
				// IStatus status = newPersp.restoreState();
				// if (status.getSeverity() != IStatus.OK) {
				// String title = WorkbenchMessages.WorkbenchPage_problemRestoringTitle;
				// String msg = WorkbenchMessages.WorkbenchPage_errorReadingState;
				// ErrorDialog.openError(getWorkbenchWindow().getShell(), title, msg, status);
				// }
			}

			// Deactivate the old layout
			if (oldPerspective != null) {
				oldPerspective.onDeactivate();

				// Notify listeners of deactivation
				// window.firePerspectiveDeactivated(this, oldPersp.getDesc());
			}

			// Activate the new layout
			perspList.setActive(newPerspective);
			if (newPerspective != null) {
				newPerspective.onActivate();

				// Notify listeners of activation
				// window.firePerspectiveActivated(this, newPersp.getDesc());
			}

			// updateVisibility(oldPersp, newPersp);

			// Update the window
			// window.updateActionSets();
		}
		finally {
			// window.largeUpdateEnd();
			if (newPerspective == null) {
				return;
			}

			IPerspectiveDescriptor desc = newPerspective.getDescriptor();
			if (desc == null) {
				return;
			}

			// if (dirtyPerspectives.remove(desc.getId())) {
			// suggestReset();
			// }
		}
	}

	private Perspective createPerspective(PerspectiveDescriptor descriptor, boolean firePerspectiveOpened) {
		Perspective persp = null;

		try {
			persp = new Perspective(descriptor, this);

		}

		catch (Exception e) {
			WorkbenchPlugin.log(e);
		}

		perspList.add(persp);

		if (firePerspectiveOpened) {
			// window.firePerspectiveOpened(this, desc);
		}

		// if the perspective is fresh and uncustomzied then it is not dirty
		// no reset will be prompted for
		// if (!desc.hasCustomDefinition()) {
		// dirtyPerspectives.remove(desc.getId());
		// }

		return persp;
	}

	public IViewPart findView(String id) {
		IViewReference ref = findViewReference(id);
		if (ref == null) {
			return null;
		}
		return ref.getView(true);
	}

	public IViewReference findViewReference(String viewId) {
		return findViewReference(viewId, null);
	}

	public IViewReference findViewReference(String viewId, String secondaryId) {
		Perspective persp = getActivePerspective();
		if (persp == null) {
			return null;
		}
		return persp.findView(viewId, secondaryId);
	}

	public Perspective findPerspective(IPerspectiveDescriptor targetPerspectiveDescriptor) {

		if (targetPerspectiveDescriptor == null) return null;

		String targetPerspectiveId = targetPerspectiveDescriptor.getId();

		if (targetPerspectiveId == null) return null;

		for (Perspective perspective : perspList) {
			if (targetPerspectiveId.equals(perspective.getDescriptor().getId())) {
				return perspective;
			}
		}

		return null;
	}

	private class PerspectiveList implements Iterable<Perspective> {
		private List<Perspective> perspectivesInOrderOpened;

		private List<Perspective> perspectivesInOrderUsed;

		private Perspective active;

		public PerspectiveList() {
			perspectivesInOrderOpened = new ArrayList<Perspective>();
			perspectivesInOrderUsed = new ArrayList<Perspective>();
		}

		public void add(Perspective perspective) {
			perspectivesInOrderOpened.add(perspective);
			perspectivesInOrderUsed.add(0, perspective);
		}

		public Perspective getActive() {
			return active;
		}

		/**
		 * Returns an iterator on the perspective list in the order they were opened.
		 */
		public Iterator<Perspective> iterator() {
			return perspectivesInOrderOpened.iterator();
		}

		/**
		 * Swap the opened order of old perspective with the new perspective.
		 */
		public void swap(Perspective oldPerspective, Perspective newPerspective) {
			int oldIndex = perspectivesInOrderOpened.indexOf(oldPerspective);
			int newIndex = perspectivesInOrderOpened.indexOf(newPerspective);

			if (oldIndex < 0 || newIndex < 0) {
				return;
			}

			perspectivesInOrderOpened.set(oldIndex, newPerspective);
			perspectivesInOrderOpened.set(newIndex, oldPerspective);
		}

		public void setActive(Perspective perspective) {
			if (perspective == active) {
				return;
			}

			// updateActionSets(active, perspective);
			active = perspective;

			if (perspective != null) {
				perspectivesInOrderUsed.remove(perspective);
				perspectivesInOrderUsed.add(perspective);
			}
		}

		/**
		 * Returns the next most recently used perspective in the list.
		 */
		public Perspective getNextActive() {
			if (active == null) {
				if (perspectivesInOrderUsed.isEmpty()) {
					return null;
				}
				return perspectivesInOrderUsed.get(perspectivesInOrderUsed.size() - 1);

			}

			if (perspectivesInOrderUsed.size() < 2) {
				return null;
			}
			return perspectivesInOrderUsed.get(perspectivesInOrderUsed.size() - 2);

		}

		/**
		 * Removes a perspective from the list.
		 */
		public boolean remove(Perspective perspective) {
			if (active == perspective) {
				// updateActionSets(active, null);
				active = null;
			}
			perspectivesInOrderUsed.remove(perspective);
			return perspectivesInOrderOpened.remove(perspective);
		}
	}

	public IWorkbenchPart getActivePart() {

		IWorkbenchPartReference activePartReference = getActivePartReference();

		if (activePartReference == null) return null;

		return activePartReference.getPart(true);
	}

	public IWorkbenchPartReference getActivePartReference() {
		return activePartRef;
	}

	public ISelection getSelection() {
		return selectionService.getSelection();
	}

	public ISelection getSelection(String partId) {
		return selectionService.getSelection(partId);
	}

	public IWorkbenchWindow getWorkbenchWindow() {
		// TODO write this in v4.0

		throw new UnsupportedOperationException();
	}

	public void addSelectionListener(ISelectionListener listener) {
		selectionService.addSelectionListener(listener);
	}

	/*
	 * (non-Javadoc) Method declared on ISelectionListener.
	 */
	public void addSelectionListener(String partId, ISelectionListener listener) {
		selectionService.addSelectionListener(partId, listener);
	}

	/*
	 * (non-Javadoc) Method declared on ISelectionListener.
	 */
	public void addPostSelectionListener(ISelectionListener listener) {
		selectionService.addPostSelectionListener(listener);
	}

	/*
	 * (non-Javadoc) Method declared on ISelectionListener.
	 */
	public void addPostSelectionListener(String partId, ISelectionListener listener) {
		selectionService.addPostSelectionListener(partId, listener);
	}

	/*
	 * (non-Javadoc) Method declared on ISelectionListener.
	 */
	public void removeSelectionListener(ISelectionListener listener) {
		selectionService.removeSelectionListener(listener);
	}

	/*
	 * (non-Javadoc) Method declared on ISelectionListener.
	 */
	public void removeSelectionListener(String partId, ISelectionListener listener) {
		selectionService.removeSelectionListener(partId, listener);
	}

	/*
	 * (non-Javadoc) Method declared on ISelectionListener.
	 */
	public void removePostSelectionListener(ISelectionListener listener) {
		selectionService.removePostSelectionListener(listener);
	}

	/*
	 * (non-Javadoc) Method declared on ISelectionListener.
	 */
	public void removePostSelectionListener(String partId, ISelectionListener listener) {
		selectionService.removePostSelectionListener(partId, listener);
	}

	/**
	 * Adds an IPartListener to the part service.
	 */
	public void addPartListener(IPartListener l) {
		// partList.getPartService().addPartListener(l);

		throw new UnsupportedOperationException();
	}

	public void resetPerspective() {
		BusyIndicator.showWhile(null, new Runnable() {
			public void run() {
				busyResetPerspective();
			}
		});
	}

	public boolean isZoomed() {
		// TODO: write this in v4.0

		return false;
	}

	public void zoomOut() {
		// TODO: write this in v4.0
	}

	/**
	 * Resets the layout for the perspective. The active part in the old layout is activated in the new layout for
	 * consistent user context.
	 * 
	 * Assumes the busy cursor is active.
	 */
	void busyResetPerspective() {

		// ViewIntroAdapterPart introViewAdapter = ((WorkbenchIntroManager)
		// getWorkbenchWindow().getWorkbench().getIntroManager()).getViewIntroAdapterPart();
		// PartPane introPane = null;
		// boolean introFullScreen = false;
		// if (introViewAdapter != null) {
		// introPane = ((PartSite) introViewAdapter.getSite()).getPane();
		// introViewAdapter.setHandleZoomEvents(false);
		// introFullScreen = introPane.isZoomed();
		// }
		//
		// // try to prevent intro flicker.
		// if (introFullScreen) {
		// window.getShell().setRedraw(false);
		// }

		try {

			// Always unzoom
			if (isZoomed()) {
				zoomOut();
			}

			// Get the current perspective.
			// This describes the working layout of the page and differs from
			// the original template.
			Perspective oldPersp = getActivePerspective();

			// Map the current perspective to the original template.
			// If the original template cannot be found then it has been deleted.
			// In that case just return. (PR#1GDSABU).
			PerspectiveRegistry reg = Workbench.getInstance().getPerspectiveRegistry();
			PerspectiveDescriptor desc = reg.findPerspectiveWithId(oldPersp.getDescriptor().getId());
			if (desc == null) {
				desc = reg.findPerspectiveWithId((oldPersp.getDescriptor()).getOriginalId());
			}
			if (desc == null) {
				return;
			}

			// Notify listeners that we are doing a reset.
			// window.firePerspectiveChanged(this, desc, CHANGE_RESET);

			// Create new persp from original template.
			// Suppress the perspectiveOpened and perspectiveClosed events otherwise it looks like two
			// instances of the same perspective are open temporarily (see bug 127470).
			Perspective newPersp = createPerspective(desc, false);
			if (newPersp == null) {
				// We're not going through with the reset, so it is complete.
				// window.firePerspectiveChanged(this, desc, CHANGE_RESET_COMPLETE);
				return;
			}

			// Update the perspective list and shortcut
			perspList.swap(oldPersp, newPersp);

			// Install new persp.
			setPerspective(newPersp);

			// Destroy old persp.
			disposePerspective(oldPersp, false);

			// Update the Coolbar layout.
			// resetToolBarLayout();

			// restore the maximized intro
			// if (introViewAdapter != null) {
			// try {
			// // ensure that the intro is visible in the new perspective
			// showView(IIntroConstants.INTRO_VIEW_ID);
			// if (introFullScreen) {
			// toggleZoom(introPane.getPartReference());
			// }
			// }
			// catch (PartInitException e) {
			// WorkbenchPlugin.log("Could not restore intro", //$NON-NLS-1$
			// WorkbenchPlugin.getStatus(e));
			// }
			// finally {
			// // we want the intro back to a normal state before we fire the event
			// introViewAdapter.setHandleZoomEvents(true);
			// }
			// }
			// Notify listeners that we have completed our reset.
			// window.firePerspectiveChanged(this, desc, CHANGE_RESET_COMPLETE);
		}
		finally {
			// reset the handling of zoom events (possibly for the second time) in case there was
			// an exception thrown
			// if (introViewAdapter != null) {
			// introViewAdapter.setHandleZoomEvents(true);
			// }
			//
			// if (introFullScreen) {
			// window.getShell().setRedraw(true);
			// }
		}

	}

	/**
	 * Dispose a perspective.
	 * 
	 * @param persp
	 *            the perspective descriptor
	 * @param notify
	 *            whether to fire a perspective closed event
	 */
	private void disposePerspective(Perspective persp, boolean notify) {
		// Get rid of perspective.
		perspList.remove(persp);
		if (notify) {
			// window.firePerspectiveClosed(this, persp.getDesc());
		}
		persp.dispose();

		// stickyPerspectives.remove(persp.getDesc().getId());
	}

	public void removePartListener(IPartListener l) {
		// partList.getPartService().removePartListener(l);

		throw new UnsupportedOperationException();
	}

	public IStatus saveState(IMemento memento) {
		MultiStatus result;

		//
		{
			String msg = MessageFormat.format(WorkbenchMessages.UnableToSavePerspective, getLabel());
			result = new MultiStatus(PlatformUI.PLUGIN_ID, IStatus.OK, msg, null);
		}

		// Save editor manager.
		IMemento childMem = memento.createChild(IWorkbenchConstants.TAG_EDITORS);

		if (editorManager != null) {
			result.merge(editorManager.saveState(childMem));
		}

		childMem = memento.createChild(IWorkbenchConstants.TAG_VIEWS);
		result.merge(getViewFactory().saveState(childMem));

		// Create persp block.
		childMem = memento.createChild(IWorkbenchConstants.TAG_PERSPECTIVES);
		if (getPerspective() != null) {
			childMem.putString(IWorkbenchConstants.TAG_ACTIVE_PERSPECTIVE, getPerspective().getId());
		}
		if (getActivePart() != null) {
			if (getActivePart() instanceof IViewPart) {
				IViewReference ref = (IViewReference) getReference(getActivePart());
				if (ref != null) {
					childMem.putString(IWorkbenchConstants.TAG_ACTIVE_PART, ViewFactory.getKey(ref));
				}
			}
			else {
				childMem.putString(IWorkbenchConstants.TAG_ACTIVE_PART, getActivePart().getSite().getId());
			}
		}

		// Save each perspective in opened order
		for (Perspective perspective : this.perspList) {
			IMemento gChildMem = childMem.createChild(IWorkbenchConstants.TAG_PERSPECTIVE);
			result.merge(perspective.saveState(gChildMem));
		}

		// // Save working set if set
		// if (workingSet != null) {
		// memento.putString(IWorkbenchConstants.TAG_WORKING_SET, workingSet.getName());
		// }

		// IMemento workingSetMem = memento.createChild(IWorkbenchConstants.TAG_WORKING_SETS);
		// for (int i = 0; i < workingSets.length; i++) {
		// workingSetMem.createChild(IWorkbenchConstants.TAG_WORKING_SET, workingSets[i].getName());
		// }
		//
		// if (aggregateWorkingSetId != null) {
		// memento.putString(ATT_AGGREGATE_WORKING_SET_ID, aggregateWorkingSetId);
		// }

		// navigationHistory.saveState(memento.createChild(IWorkbenchConstants.TAG_NAVIGATION_HISTORY));

		// IMemento stickyState = memento.createChild(IWorkbenchConstants.TAG_STICKY_STATE);
		//		
		// // save the sticky activation state
		// itr = stickyPerspectives.entrySet().iterator();
		// while (itr.hasNext()) {
		// Map.Entry entry = (Map.Entry) itr.next();
		// String perspectiveId = (String) entry.getKey();
		// Set activatedViewIds = (Set) entry.getValue();
		// IMemento perspectiveState = stickyState.createChild(IWorkbenchConstants.TAG_PERSPECTIVE, perspectiveId);
		// for (Iterator i = activatedViewIds.iterator(); i.hasNext();) {
		// String viewId = (String) i.next();
		// perspectiveState.createChild(IWorkbenchConstants.TAG_VIEW, viewId);
		// }
		// }

		return result;
	}

	public IStatus restoreState(IMemento memento, IPerspectiveDescriptor activeDescriptor) {

		// Restore working set
		String pageName = memento.getString(IWorkbenchConstants.TAG_LABEL);

		if (pageName == null) {
			pageName = ""; //$NON-NLS-1$
		}
		MultiStatus result = new MultiStatus(PlatformUI.PLUGIN_ID, IStatus.OK, NLS.bind(
				WorkbenchMessages.WorkbenchPage_unableToRestorePerspective, pageName), null);

		// String workingSetName = memento
		// .getString(IWorkbenchConstants.TAG_WORKING_SET);
		// if (workingSetName != null) {
		// WorkingSetManager workingSetManager = (WorkingSetManager) getWorkbenchWindow()
		// .getWorkbench().getWorkingSetManager();
		// setWorkingSet(workingSetManager.getWorkingSet(workingSetName));
		// }
		//	                
		// IMemento workingSetMem = memento
		// .getChild(IWorkbenchConstants.TAG_WORKING_SETS);
		// if (workingSetMem != null) {
		// IMemento[] workingSetChildren = workingSetMem
		// .getChildren(IWorkbenchConstants.TAG_WORKING_SET);
		// List workingSetList = new ArrayList(
		// workingSetChildren.length);
		// for (int i = 0; i < workingSetChildren.length; i++) {
		// IWorkingSet set = getWorkbenchWindow().getWorkbench()
		// .getWorkingSetManager().getWorkingSet(
		// workingSetChildren[i].getID());
		// if (set != null) {
		// workingSetList.add(set);
		// }
		// }
		//
		// workingSets = (IWorkingSet[]) workingSetList
		// .toArray(new IWorkingSet[workingSetList.size()]);
		// }
		//		            
		// aggregateWorkingSetId = memento.getString(ATT_AGGREGATE_WORKING_SET_ID);
		//		            
		// IWorkingSet setWithId = window.getWorkbench().getWorkingSetManager().getWorkingSet(aggregateWorkingSetId);
		//		            
		// // check to see if the set has already been made and assign it if it has
		// if (setWithId instanceof AggregateWorkingSet) {
		// aggregateWorkingSet = (AggregateWorkingSet) setWithId;
		// }
		// Restore editor manager.
		IMemento childMem = memento.getChild(IWorkbenchConstants.TAG_EDITORS);
		// result.merge(getEditorManager().restoreState(childMem));

		childMem = memento.getChild(IWorkbenchConstants.TAG_VIEWS);
		if (childMem != null) {
			result.merge(getViewFactory().restoreState(childMem));
		}

		// Get persp block.
		childMem = memento.getChild(IWorkbenchConstants.TAG_PERSPECTIVES);
		String activePartID = childMem.getString(IWorkbenchConstants.TAG_ACTIVE_PART);
		String activePartSecondaryID = null;
		if (activePartID != null) {
			activePartSecondaryID = ViewFactory.extractSecondaryId(activePartID);
			if (activePartSecondaryID != null) {
				activePartID = ViewFactory.extractPrimaryId(activePartID);
			}
		}
		String activePerspectiveID = childMem.getString(IWorkbenchConstants.TAG_ACTIVE_PERSPECTIVE);

		// Restore perspectives.
		IMemento perspMems[] = childMem.getChildren(IWorkbenchConstants.TAG_PERSPECTIVE);
		Perspective activePerspective = null;
		for (int i = 0; i < perspMems.length; i++) {
			try {
				Perspective persp = new Perspective(null, this);
				result.merge(persp.restoreState(perspMems[i]));
				IPerspectiveDescriptor desc = persp.getDescriptor();
				if (desc.equals(activeDescriptor)) {
					activePerspective = persp;
				}
				else if ((activePerspective == null) && desc.getId().equals(activePerspectiveID)) {
					activePerspective = persp;
				}
				perspList.add(persp);
				// window.firePerspectiveOpened(this, desc);
			}
			catch (WorkbenchException e) {
			}
		}
		boolean restoreActivePerspective = false;
		if (activeDescriptor == null) {
			restoreActivePerspective = true;
		}
		else if (activePerspective != null && activePerspective.getDescriptor().equals(activeDescriptor)) {
			restoreActivePerspective = true;
		}
		else {
			restoreActivePerspective = false;
			activePerspective = createPerspective((PerspectiveDescriptor) activeDescriptor, true);
			if (activePerspective == null) {
				result.merge(new Status(IStatus.ERROR, PlatformUI.PLUGIN_ID, 0, NLS.bind(
						WorkbenchMessages.Workbench_showPerspectiveError, activeDescriptor.getId()), null));
			}
		}

		perspList.setActive(activePerspective);

		// Make sure we have a valid perspective to work with,
		// otherwise return.
		activePerspective = perspList.getActive();
		if (activePerspective == null) {
			activePerspective = perspList.getNextActive();
			perspList.setActive(activePerspective);
			result.merge(activePerspective.restoreState());
		}

		if (activePerspective != null && restoreActivePerspective) {
			result.merge(activePerspective.restoreState());
		}

		if (activePerspective != null) {
			// window.firePerspectiveActivated(this, activePerspective.getDesc());

			// Restore active part.
			if (activePartID != null) {
				IViewReference ref = activePerspective.findView(activePartID, activePartSecondaryID);

				if (ref != null) {
					// activationList.setActive(ref);
				}
			}
		}

		childMem = memento.getChild(IWorkbenchConstants.TAG_NAVIGATION_HISTORY);
		// if (childMem != null) {
		// navigationHistory.restoreState(childMem);
		// } else if (getActiveEditor() != null) {
		// navigationHistory.markEditor(getActiveEditor());
		// }

		// IMemento stickyState = memento.getChild(IWorkbenchConstants.TAG_STICKY_STATE);
		// restore the sticky activation state
		//
		// if (stickyState != null) {
		// IMemento[] stickyPerspMems = stickyState
		// .getChildren(IWorkbenchConstants.TAG_PERSPECTIVE);
		// for (int i = 0; i < stickyPerspMems.length; i++) {
		// String perspectiveId = stickyPerspMems[i].getID();
		// Set viewState = new HashSet(7);
		// stickyPerspectives.put(perspectiveId, viewState);
		// IMemento[] viewStateMementos = stickyPerspMems[i]
		// .getChildren(IWorkbenchConstants.TAG_VIEW);
		// for (int j = 0; j < viewStateMementos.length; j++) {
		// viewState.add(viewStateMementos[j].getID());
		// }
		// }
		//
		// }
		return result;

	}

	public IPerspectiveDescriptor getPerspective() {
		if (deferredActivePerspective != null) {
			return deferredActivePerspective;
		}

		Perspective perspective = getActivePerspective();
		if (perspective != null) {
			return perspective.getDescriptor();
		}

		return null;
	}

	public String getLabel() {

		String label = WorkbenchMessages.UnknownLabel;

		// TODO: write this in v4.0

		return label;
	}

	public List<IViewReference> getViewReferences() {
		Perspective persp = getActivePerspective();
		if (persp == null) {
			return Collections.emptyList();
		}

		return persp.getViewReferences();
	}

	public DefaultDockingManager getDockingManager() {
		return getComponent().getDockingManager();
	}

    static class KeyboardFocusManagerPropertyChangeListener implements PropertyChangeListener {
            public void propertyChange(PropertyChangeEvent event) {

                    try {
                            Object newValue = event.getNewValue();

                            if (!(newValue instanceof Component)) return;

                            Component component = (Component) newValue;

                            PartPane partPane = WorkbenchPageUtils.getPartPane(component);

                            if (partPane == null) return;

                            WorkbenchPage page = partPane.getPage();

                            IWorkbenchPartReference partRef = partPane.getPartReference();

                            IWorkbenchPart part = partRef.getPart(true);

                            page.selectionService.setActivePart(part);

                            page.activePartRef = partRef;
                    }
                    catch (Throwable t) {
                            WorkbenchPlugin.log(t);
                    }
            }

    }

    static public class WorkbenchPageComponent extends JPanel {
            private static final long serialVersionUID = 1L;

            private DefaultDockingManager dockingManager;

            private DocumentPane documentPane;

            public WorkbenchPageComponent() {
                    initialize();
            }

            public final DocumentPane getDocumentPane() {
                    if (documentPane == null) {
                            documentPane = new DocumentPane();
                            documentPane.setTabbedPaneCustomizer(new DocumentPaneTabbedPaneCustomizer());
                    }

                    return documentPane;
            }

            private void initialize() {
                    this.setLayout(new BorderLayout());
            }

            public final DefaultDockingManager getDockingManager() {
                    if (dockingManager == null) {

                            Window windowAncestor = SwingUtilities.getWindowAncestor(this);

                            RootPaneContainer rootContainer = (RootPaneContainer) windowAncestor;

                            dockingManager = new DefaultDockingManager(rootContainer, this);

                            dockingManager.setAutohidable(false);
                            dockingManager.setFloatable(false);
                            dockingManager.setAutoActivateFocusOwner(true);
                            dockingManager.setFocusDuringLayout(true);
                            dockingManager.setUseGlassPaneEnabled(true);

                            dockingManager.setTabbedPaneCustomizer(new DockingManagerTabbedPaneCustomizer());
                            dockingManager.setDoubleClickAction(DefaultDockingManager.DOUBLE_CLICK_TO_MAXIMIZE);
                            dockingManager.setProportionalSplits(true);
                            dockingManager.setShowTitleBar(false);
                            dockingManager.setTabDockAllowed(true);
                            dockingManager.setEasyTabDock(true);

                            // dockingManager.setUseDecoratedFloatingContainer(true);

                            dockingManager.setUseFrameState(false);
                            dockingManager.setUseFrameBounds(false);
                            dockingManager.setUsePref(false);

                            dockingManager.getWorkspace().setLayout(new BorderLayout());
                            dockingManager.getWorkspace().add(getDocumentPane(), BorderLayout.CENTER);

                            dockingManager.resetToDefault();
                    }

                    return dockingManager;
            }

            public void dispose() {
                    if (dockingManager != null) {
                            dockingManager.dispose();
                    }
            }

        static class DockingManagerTabbedPaneCustomizer implements DockingManager.TabbedPaneCustomizer {
            public void customize(JideTabbedPane tabbedPane) {

                tabbedPane.setBorder(new LineBorder(Color.LIGHT_GRAY, 1, true));
                tabbedPane.setTabPlacement(JideTabbedPane.TOP);
                tabbedPane.setTabShape(JideTabbedPane.SHAPE_OFFICE2003);

                tabbedPane.setHideOneTab(false);

                tabbedPane.setShowCloseButtonOnTab(true);
                tabbedPane.setShowCloseButton(true);
                tabbedPane.setShowCloseButtonOnSelectedTab(true);

//		tabbedPane.setTabTrailingComponent(new DockingManagerTabTrailingComponent());
            }

            class DockingManagerTabTrailingComponent extends JToolBar implements UIResource {

                private static final long serialVersionUID = 1L;

                public DockingManagerTabTrailingComponent() {
                    this.setFloatable(false);
                    this.setOpaque(false);

                    add(new JButton("X")); //$NON-NLS-1$
                }
            }
        }

        class DocumentPaneTabbedPaneCustomizer implements DocumentPane.TabbedPaneCustomizer {

            public void customize(JideTabbedPane tabbedPane) {

                // tabbedPane.setBoldActiveTab(false);
                // tabbedPane.setBoxStyleTab(false);
            }

        }

    }

}




