/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.onaro.client.leekui.ui.PlatformUI;
import com.onaro.client.leekui.ui.internal.util.BundleUtility;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.WorkbenchPlugin
 */
public class WorkbenchPlugin {

	/**
	 * The workbench plugin ID.
	 * 
	 * @issue we should just drop this constant and use PlatformUI.PLUGIN_ID instead
	 */
	public static String PI_WORKBENCH = PlatformUI.PLUGIN_ID;

	static final Logger logger = Logger.getLogger(WorkbenchPlugin.class);

	/**
	 * Creates an extension. If the extension plugin has not been loaded a busy cursor will be activated during the
	 * duration of the load.
	 * 
	 * @param element
	 *            the config element defining the extension
	 * @param classAttribute
	 *            the name of the attribute carrying the class
	 * @return the extension object
	 * @throws CoreException
	 *             if the extension cannot be created
	 */
	public static Object createExtension(final IConfigurationElement element, final String classAttribute)
			throws CoreException {
		try {
			// If plugin has been loaded create extension.
			// Otherwise, show busy cursor then create extension.
			if (BundleUtility.isActivated(element.getDeclaringExtension().getNamespace())) {
				return element.createExecutableExtension(classAttribute);
			}
			final Object[] ret = new Object[1];
			final CoreException[] exc = new CoreException[1];
			BusyIndicator.showWhile(null, new Runnable() {
				public void run() {
					try {
						ret[0] = element.createExecutableExtension(classAttribute);
					} catch (CoreException e) {
						exc[0] = e;
					}
				}
			});
			if (exc[0] != null) {
				throw exc[0];
			}
			return ret[0];

		} catch (CoreException core) {
			throw core;
		} catch (Exception e) {
			throw new CoreException(new Status(IStatus.ERROR, PI_WORKBENCH, IStatus.ERROR,
					WorkbenchMessages.WorkbenchPlugin_extension, e));
		}
	}

	public static void log(Throwable t) {

		// TODO: re-write this in v4.0
		logger.error( t.getMessage(), t);
	}

	/**
	 * Logs the given message to the platform log.
	 * 
	 * If you have an exception in hand, call log(String, Throwable) instead.
	 * 
	 * If you have a status object in hand call log(String, IStatus) instead.
	 * 
	 * This convenience method is for internal use by the Workbench only and must not be called outside the Workbench.
	 * 
	 * @param message
	 *            A high level UI message describing when the problem happened.
	 */
	public static void log(String message) {

		// TODO: re-write this in v4.0

		// getDefault().getLog().log(
		// StatusUtil.newStatus(IStatus.ERROR, message, null));

		logger.error(message);
	}

	/**
	 * Logs the given message and throwable to the platform log.
	 * 
	 * If you have a status object in hand call log(String, IStatus) instead.
	 * 
	 * This convenience method is for internal use by the Workbench only and must not be called outside the Workbench.
	 * 
	 * @param message
	 *            A high level UI message describing when the problem happened.
	 * @param t
	 *            The throwable from where the problem actually occurred.
	 */
	public static void log(String message, Throwable t) {
		// TODO: re-write this in v4.0

		logger.error(message, t);
	}
}
