/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.compatibility;

import javax.swing.Icon;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.onaro.client.leekui.ui.IMemento;
import com.onaro.client.leekui.ui.IPerspectiveDescriptor;
import com.onaro.client.leekui.ui.IPerspectiveFactory;
import com.onaro.client.leekui.ui.PlatformUI;
import com.onaro.client.leekui.ui.internal.IWorkbenchConstants;
import com.onaro.client.leekui.ui.internal.Workbench;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.registry.PerspectiveDescriptor
 */
public class PerspectiveDescriptor implements IPerspectiveDescriptor {
	private Class<? extends IPerspectiveFactory> perspectiveFactoryClass;

	private String id;

	private String label;

	private boolean fixed;

	private String originalId;

	private boolean singleton;

	public PerspectiveDescriptor(Class<? extends IPerspectiveFactory> perspectiveFactoryClass, String id, String label,
			boolean fixed) {
		this.perspectiveFactoryClass = perspectiveFactoryClass;
		this.id = id;
		this.label = label;
		this.fixed = fixed;
	}

	public boolean getFixed() {
		return fixed;
	}

	public String getDescription() {
		// TODO write this in v4.0

		return null;
	}

	public String getId() {
		return id;
	}

	public Icon getIcon() {
		// TODO write this in v4.0

		return null;
	}

	public String getLabel() {
		return label;
	}

	public boolean hasCustomDefinition() {
		return false;
	}

	public IPerspectiveFactory createFactory() throws Exception {

		IPerspectiveFactory perspectiveFactory = perspectiveFactoryClass.newInstance();

		return perspectiveFactory;
	}

	/**
	 * Restore the state of a perspective from a memento.
	 * 
	 * @param memento
	 *            the memento to restore from
	 * @return the <code>IStatus</code> of the operation
	 * 
	 */
	public IStatus restoreState(IMemento memento) {
		IMemento childMem = memento.getChild(IWorkbenchConstants.TAG_DESCRIPTOR);
		if (childMem != null) {
			id = childMem.getString(IWorkbenchConstants.TAG_ID);
			originalId = childMem.getString(IWorkbenchConstants.TAG_DESCRIPTOR);
			label = childMem.getString(IWorkbenchConstants.TAG_LABEL);
			// className = childMem.getString(IWorkbenchConstants.TAG_CLASS);
			singleton = (childMem.getInteger(IWorkbenchConstants.TAG_SINGLETON) != null);

			// Find a descriptor in the registry.
			PerspectiveDescriptor descriptor = Workbench.getInstance().getPerspectiveRegistry().findPerspectiveWithId(
					getOriginalId());

			if (descriptor != null) {
				// Copy the state from the registred descriptor.
				// image = descriptor.getImageDescriptor();
				perspectiveFactoryClass = descriptor.perspectiveFactoryClass;
			}
		}
		return new Status(IStatus.OK, PlatformUI.PLUGIN_ID, 0, "", null); //$NON-NLS-1$
	}

	public IStatus saveState(IMemento memento) {
		IMemento childMem = memento.createChild(IWorkbenchConstants.TAG_DESCRIPTOR);
		childMem.putString(IWorkbenchConstants.TAG_ID, getId());
		if (originalId != null) {
			childMem.putString(IWorkbenchConstants.TAG_DESCRIPTOR, originalId);
		}
		childMem.putString(IWorkbenchConstants.TAG_LABEL, getLabel());
		childMem.putString(IWorkbenchConstants.TAG_CLASS, getClassName());
		if (singleton) {
			childMem.putInteger(IWorkbenchConstants.TAG_SINGLETON, 1);
		}
		return new Status(IStatus.OK, PlatformUI.PLUGIN_ID, 0, "", null); //$NON-NLS-1$
	}

	public String getClassName() {
		return perspectiveFactoryClass.getName();
	}

	public String getOriginalId() {
		if (originalId == null) {
			return getId();
		}
		return originalId;
	}
}
