/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.tree.TreeModel;

import org.apache.commons.lang.StringUtils;

import com.jidesoft.tree.QuickTreeFilterField;
import com.jidesoft.tree.TreeUtils;
import com.onaro.client.leekui.ui.views.IViewDescriptor;

public class ShowViewDialogContent extends JPanel {
	private static final long serialVersionUID = 1L;

	private TreeModel viewRegistryTreeModel;

	private ViewRegistryTree viewRegistryTree;

	private TreeModel displayTreeModel;

	private QuickTreeFilterField quickTreeFilterField;

	private JScrollPane scrollPane;
	
	private final String ancestorCategoryId;
	
	public ShowViewDialogContent() { 
		this(null);
	}

	/**
	 * 
	 * @param ancestorCategoryId (Optional) Only show descendent views of this categoryId.
	 */
	public ShowViewDialogContent(String ancestorCategoryId) {
		super();
		
		this.ancestorCategoryId = StringUtils.trimToNull(ancestorCategoryId);

		initialize();
	}

	private void initialize() {
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.fill = GridBagConstraints.BOTH;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.ipadx = 0;
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		gridBagConstraints1.gridx = 0;
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.insets = new Insets(0, 0, 5, 0);
		gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints.gridy = 0;
		setLayout(new GridBagLayout());
		this.add(getQuickTreeFilterField(), gridBagConstraints);
		this.add(getScrollPane(), gridBagConstraints1);

	}

	private QuickTreeFilterField getQuickTreeFilterField() {
		if (quickTreeFilterField == null) {
			quickTreeFilterField = new QuickTreeFilterField(getViewRegistryTreeModel());
			//these values are taken from eclipse
			quickTreeFilterField.setPreferredSize(new Dimension(230, 24));
			quickTreeFilterField.setTree(getViewRegistryTree());
		}

		return quickTreeFilterField;
	}

	private TreeModel getDisplayTreeModel() {
		if (displayTreeModel == null) {
			displayTreeModel = getQuickTreeFilterField().getDisplayTreeModel();
		}

		return displayTreeModel;
	}

	private JScrollPane getScrollPane() {
		if (scrollPane == null) {
			scrollPane = new JScrollPane();
			scrollPane.setViewportView(getViewRegistryTree());
		}

		return scrollPane;
	}

	private ViewRegistryTree getViewRegistryTree() {
		if (viewRegistryTree == null) {
			viewRegistryTree = new ViewRegistryTree(getDisplayTreeModel());
			TreeUtils.expandAll(viewRegistryTree);
		}

		return viewRegistryTree;
	}

	private TreeModel getViewRegistryTreeModel() {
		if (viewRegistryTreeModel == null) {
			viewRegistryTreeModel = ViewRegistryTree.createModel(ancestorCategoryId);
		}

		return viewRegistryTreeModel;
	}

	public IViewDescriptor getSelectedViewDescriptor()
	{
		return getViewRegistryTree().getSelectedViewDescriptor();
	}
	
	public List<IViewDescriptor> getSelectedViewDescriptors() {
		
		return getViewRegistryTree().getSelectedViewDescriptors();
	}

}
