/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.dialogs;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

import com.onaro.client.leekui.jface.preferences.PreferenceManager;
import com.onaro.client.leekui.ui.internal.DashboardPlugin;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.dialogs.WorkbenchPreferenceManager
 */
public class WorkbenchPreferenceManager extends PreferenceManager {
	
	static final Logger logger = Logger.getLogger(WorkbenchPreferenceManager.class);

	public WorkbenchPreferenceManager() {
		loadPagesFromExtensions();
	}
	
	private void loadPagesFromExtensions() {
		IExtensionRegistry extensionRegistry = Platform.getExtensionRegistry();

		IExtensionPoint prefExtensionPoint = extensionRegistry.getExtensionPoint(DashboardPlugin.PLUGIN_ID + ".preferencePages"); //$NON-NLS-1$

		if (prefExtensionPoint != null) {
			IConfigurationElement[] configurationElements = prefExtensionPoint.getConfigurationElements(); 

			for (final IConfigurationElement configurationElement : configurationElements) {

				WorkbenchPreferenceNode preferenceNode = new WorkbenchPreferenceNode(configurationElement.getAttribute("id"), configurationElement); //$NON-NLS-1$
				String path = configurationElement.getAttribute("category"); //$NON-NLS-1$
				if (path == null || path.trim().length() == 0) {
					path = "/"; //$NON-NLS-1$
				}
				addTo(path, preferenceNode);
			}
		}
	}
}
