/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.progress;

import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.ProgressProvider;

import com.jidesoft.status.StatusBarItem;
import com.onaro.client.leekui.jface.operation.IRunnableWithProgress;
import com.onaro.client.leekui.jface.operation.IUIRunnableWithProgress;
import com.onaro.client.leekui.ui.progress.IProgressService;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.progress.ProgressManager
 */
public final class ProgressManager extends ProgressProvider implements IProgressService {
	
	static final Logger logger = Logger.getLogger(ProgressManager.class);

	private static final ProgressManager INSTANCE = new ProgressManager();
	
	public static ProgressManager getInstance() {
		return INSTANCE;
	}
	
	private static final int DEFAULT_LONG_OPERATION_TIME = 1500;
	
	private int longOperationTime = DEFAULT_LONG_OPERATION_TIME;
	
	private ProgressMonitorsDialogManager progressMonitorsDialogManager;
	
	private ExecutorService executorService;
	
	private ProgressManager() {
		initialize();
	}
	
	private void initialize() {
		progressMonitorsDialogManager = new ProgressMonitorsDialogManager();
		
		Job.getJobManager().setProgressProvider(this);
	}
	
	private ExecutorService getExecutorService() {
		if (executorService == null) {
			executorService = Executors.newCachedThreadPool();
		}
		
		return executorService;
	}
	
	public void showProgressDialog() {
		progressMonitorsDialogManager.showDialog();
	}
	
	@Override
	public IProgressMonitor createMonitor(Job job) {
		return createProgressMonitor(true, true);
	}
	
	private IProgressMonitor createProgressMonitor(boolean cancelable, boolean background) {
		return progressMonitorsDialogManager.createProgressMonitor(cancelable, background);
	}
	

	public void run(boolean fork, boolean cancelable, final IRunnableWithProgress runnable) throws InvocationTargetException, InterruptedException {
		
		boolean backgroundTask = true;
		
		if (runnable instanceof IUIRunnableWithProgress) {
			backgroundTask = ((IUIRunnableWithProgress)runnable).isBackground();
		}
		
		final IProgressMonitor progressMonitor = createProgressMonitor(cancelable, backgroundTask);
		
		if (!fork) {
			runnable.run(progressMonitor);
		} else {
			Callable<?> callable = new Callable<Object>() {

				public Object call() throws Exception {
					try {
						runnable.run(progressMonitor);
					} catch (InterruptedException e) {
						// Don't log interrupted exceptions
						throw e;
					} catch (Exception e) {
						// Log the exception
						logger.warn("Exception while executing task" + " - " + e.getMessage(), e); //$NON-NLS-1$ //$NON-NLS-2$
						throw e;
					}
					return null;
				}
				
			};
			
			ExecutorService executorService = getExecutorService();
			executorService.submit(callable);
		}
	}

	public void busyCursorWhile(IRunnableWithProgress runnable) throws InvocationTargetException, InterruptedException {
		// TODO write this
	}

	public int getLongOperationTime() {
		return longOperationTime;
	}
	
	public StatusBarItem getProgressStatusBarItem() {
		return progressMonitorsDialogManager.getProgressStatusBarItem();
	}
	
}
