/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.progress;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;

import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.jidesoft.swing.JideBoxLayout;
import com.onaro.client.leekui.jface.preferences.Messages;

public class ProgressMonitorsDialog extends JDialog {

	private static final Color COMPONENT_BACKGROUND_COLOR_PRIMARY = Color.WHITE;
	private static final Color COMPONENT_BACKGROUND_COLOR_SECONDARY = Color.decode("#F5F5F5"); //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;
	
	private JScrollPane scrollPane = null;
	
	private JPanel progressMonitorsContainer;

	/**
	 * This method initializes 
	 * 
	 */
	public ProgressMonitorsDialog(Frame parent) {
		super(parent, false);
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.setSize(new Dimension(282, 100));
        this.setContentPane(getScrollPane());
        this.setTitle(Messages.INSTANCE.getProgressTitle());
        this.setAlwaysOnTop(false);
	}

	/**
	 * This method initializes scrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getScrollPane() {
		if (scrollPane == null) {
			scrollPane = new JScrollPane();
			scrollPane.setViewportView(getProgressMonitorsContainer());
		}
		return scrollPane;
	}
	
	private JPanel getProgressMonitorsContainer() {
		if (progressMonitorsContainer == null) {
			progressMonitorsContainer = new JPanel();
			JideBoxLayout layout = new JideBoxLayout(progressMonitorsContainer, JideBoxLayout.Y_AXIS);
			layout.setResetWhenInvalidate(true);
			progressMonitorsContainer.setLayout(layout);
			progressMonitorsContainer.setBackground(COMPONENT_BACKGROUND_COLOR_PRIMARY);
		}
		
		return progressMonitorsContainer;
	}
	
	public void add(ProgressMonitorComponent progressMonitorComponent) {
		if (progressMonitorComponent == null) throw new IllegalArgumentException("progressMonitorComponent"); //$NON-NLS-1$
		
		progressMonitorComponent.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		getProgressMonitorsContainer().add(progressMonitorComponent, JideBoxLayout.FIX);
		updateProgressMonitorComponents();
	}
	
	public void remove(ProgressMonitorComponent progressMonitorComponent) {
		if (progressMonitorComponent == null) throw new IllegalArgumentException("progressMonitorComponent"); //$NON-NLS-1$
		getProgressMonitorsContainer().remove(progressMonitorComponent);
		updateProgressMonitorComponents();
	}
	
	private void updateProgressMonitorComponents() {
		Component[] progressMonitorComponents = getProgressMonitorComponents();
		for (int index = 0; index < progressMonitorComponents.length; index++) {
			Color background = (index % 2 == 0) ? COMPONENT_BACKGROUND_COLOR_SECONDARY : COMPONENT_BACKGROUND_COLOR_PRIMARY;
			progressMonitorComponents[index].setBackground(background);
		}
		getProgressMonitorsContainer().validate();
		getProgressMonitorsContainer().repaint();
	}
	
	public int getProgressMonitorComponentCount() {
		return getProgressMonitorsContainer().getComponentCount();
	}
	
	private Component[] getProgressMonitorComponents() {
		return getProgressMonitorsContainer().getComponents();
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
