/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.registry;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.onaro.client.leekui.ui.IEditorDescriptor;
import com.onaro.client.leekui.ui.IEditorRegistry;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.registry.EditorRegistry
 */
public class EditorRegistry implements IEditorRegistry
{
	private final Map<String,IEditorDescriptor> editorDescriptorsByIdMap;
	
	public EditorRegistry()
	{
		editorDescriptorsByIdMap = new HashMap<String,IEditorDescriptor>();
	}

	public void add(IEditorDescriptor editorDescriptor) {
		
		if (editorDescriptor == null) throw new IllegalArgumentException();
		
		String editorId = editorDescriptor.getId();
		
		if (editorId == null) throw new IllegalArgumentException();
		
		editorDescriptorsByIdMap.put(editorId, editorDescriptor);
	}

	public IEditorDescriptor findEditor(String editorId) {
		return editorDescriptorsByIdMap.get(editorId);
	}
	
	public List<IEditorDescriptor> getEditors()
	{
		return new ArrayList<IEditorDescriptor>(editorDescriptorsByIdMap.values());
	}
}
