/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.registry;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.ObjectUtils;

import com.onaro.client.leekui.ui.IPerspectiveDescriptor;
import com.onaro.client.leekui.ui.IPerspectiveRegistry;
import com.onaro.client.leekui.ui.internal.compatibility.PerspectiveDescriptor;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.registry.PerspectiveRegistry
 */
public class PerspectiveRegistry implements IPerspectiveRegistry {

	private final Map<String,PerspectiveDescriptor> perspectiveDescriptorsByIdMap;

	public PerspectiveRegistry() {
		perspectiveDescriptorsByIdMap = new HashMap<String,PerspectiveDescriptor>();
	}

	public PerspectiveDescriptor findPerspectiveWithId(String perspectiveId)
    {
    	return perspectiveDescriptorsByIdMap.get(perspectiveId);
    }

    public IPerspectiveDescriptor findPerspectiveWithLabel(String label)
    {
    	for (IPerspectiveDescriptor perspective : getPerspectives())
    	{
    		if (ObjectUtils.equals(label, perspective.getLabel())) return perspective;
    	}
    	
    	return null;
    }
	
	public void add(PerspectiveDescriptor perspectiveDescriptor) {
		if (perspectiveDescriptor == null) throw new IllegalArgumentException();
		
		String id = perspectiveDescriptor.getId();
		
		if (id == null) throw new IllegalArgumentException();
		
		perspectiveDescriptorsByIdMap.put(id, perspectiveDescriptor);
	}

	public List<IPerspectiveDescriptor> getPerspectives()
	{
		return new ArrayList<IPerspectiveDescriptor>(perspectiveDescriptorsByIdMap.values());
	}
	
	
}
