/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.dialogs;

import javax.swing.Icon;
import javax.swing.JComponent;

import com.onaro.client.leekui.jface.dialogs.IDialogPage;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.jface.dialogs.DialogPage
 */
public abstract class DialogPage implements IDialogPage {
	
	protected JComponent component;
	protected Icon icon;
	protected String description;
	protected String title;
	protected String errorMessage;
	protected String warningMessage;
	protected String message;
	
	protected DialogPage() {
		// No initial behavior
	}
	
	protected DialogPage(String title) {
		this.title = title;
	}
	
	protected DialogPage(String title, Icon icon) {
		this(title);
		this.icon = icon;
	}
	
	protected abstract JComponent createComponent();
	
	public final JComponent getComponent(boolean create) {
		if (component == null && create) {
			component = createComponent();
		}
		return component;
	}

	public void dispose() {
		// do nothing by default
	}

	public String getDescription() {
		return description;
	}

	public String getErrorMessage() {
		return errorMessage;
	}
	
	public void setErrorMessage(String errorMessage) {
		this.errorMessage = errorMessage;
	}
	
	public String getWarningMessage() {
		return warningMessage;
	}
	
	public void setWarningMessage(String warningMessage) {
		this.warningMessage = warningMessage;
	}
	
	public String getMessage() {
		return message;
	}

	public void setMessage(String message) {
		this.message = message;
	}

	public Icon getIcon() {
		return icon;
	}
	
	public String getTitle() {
		return title;
	}

	public void performHelp() {
		// do nothing by default
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public void setIcon(Icon icon) {
		this.icon = icon;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public void setVisible(boolean visible) {
		// do nothing by default
	}

}
