/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.dialogs;

import javax.swing.Icon;
import javax.swing.JComponent;



/**
 * Interface for a page in a multi-page dialog.
 * This class is a reimplementation of the Eclipse class org.eclipse.jface.dialogs.IDialogPage
 */
public interface IDialogPage {
	
    /**
     * Returns the top level control for this dialog page.
     * <p>
     *  
     * May return <code>null</code> if the control
     * has not been created yet and the create param is false.
     * </p>
     * 
     * @param create true if the component should be created, false otherwise
     *
     * @return the top level control or <code>null</code>
     */
    public JComponent getComponent(boolean create);

    /**
     * Disposes the SWT resources allocated by this
     * dialog page.
     */
    public void dispose();

    /**
     * Returns this dialog page's description text.
     *
     * @return the description text for this dialog page, 
     *  or <code>null</code> if none
     */
    public String getDescription();

    /**
     * Returns the current error message for this dialog page.
     * May be <code>null</code> to indicate no error message.
     * <p>
     * An error message should describe some error state,
     * as opposed to a message which may simply provide instruction
     * or information to the user.
     * </p>
     * 
     * @return the error message, or <code>null</code> if none
     */
    public String getErrorMessage();
    
    /**
     * Returns the current warning message for this dialog page.
     * May be <code>null</code> to indicate no warning message.
     * <p>
     * An error warning should describe some warning state,
     * as opposed to a message which may simply provide instruction
     * or information to the user.
     * </p>
     * 
     * @return the warning message, or <code>null</code> if none
     */
    public String getWarningMessage();

    /**
     * Returns this dialog page's image.
     *
     * @return the image for this dialog page, or <code>null</code>
     *  if none
     */
    public Icon getIcon();

    /**
     * Returns the current message for this wizard page.
     * <p>
     * A message provides instruction or information to the 
     * user, as opposed to an error message which should 
     * describe some error state.
     * </p>
     * 
     * @return the message, or <code>null</code> if none
     */
    public String getMessage();

    /**
     * Returns this dialog page's title.
     *
     * @return the title of this dialog page, 
     *  or <code>null</code> if none
     */
    public String getTitle();

    /**
     * Notifies that help has been requested for this dialog page.
     */
    public void performHelp();

    /**
     * Sets this dialog page's description text.
     * 
     * @param description the description text for this dialog
     *  page, or <code>null</code> if none
     */
    public void setDescription(String description);

    /**
     * Sets this dialog page's icon.
     *
     * @param icon the icon for this dialog page, 
     *  or <code>null</code> if none
     */
    public void setIcon(Icon icon);

    /**
     * Set this dialog page's title.
     *
     * @param title the title of this dialog page, 
     *  or <code>null</code> if none
     */
    public void setTitle(String title);

    /**
     * Sets the visibility of this dialog page.
     *
     * @param visible <code>true</code> to make this page visible,
     *  and <code>false</code> to hide it
     */
    public void setVisible(boolean visible);
}
