/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.dialogs;


/**
 * Minimal interface to a message provider. Used for dialog pages which can
 * provide a message with an icon.
 * 
 * @since 2.0
 * This class is a reimplementation of the Eclipse class org.eclipse.jface.dialogs.IMessageProvider
 */
public interface IMessageProvider {
	
	public static enum Type {NONE, INFO, WARNING, ERROR};
	
    /**
     * Returns the current message for this message provider.
     * <p>
     * A message provides instruction or information to the user.
     * </p>
     * 
     * @return the message, or <code>null</code> if none
     */
    public String getMessage();

    /**
     * Returns a value indicating if the message is a an information message, a
     * warning message, or an error message.
     * <p>
     * Returns one of <code>NONE</code>,<code>INFORMATION</code>,
     * <code>WARNING</code>, or <code>ERROR</code>.
     * </p>
     * 
     * @return the message type
     */
    public Type getMessageType();
}
