/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.dialogs;

import javax.swing.Icon;
import javax.swing.JComponent;

/**
 * Simple implementation of DialogPage that contains one component.
 * The dialog page can be configured using the standard setTitle(), 
 * setError/Warning/Message() calls, and setIcon().
 * The isPageValid() method can be overriden to add validity checking.
 *
 */
public class SimpleDialogPage extends ValidatableDialogPage {
	
	private JComponent component;
	
	public SimpleDialogPage(JComponent component) {
		this(component, null);
	}
	
	public SimpleDialogPage(JComponent component, String title) {
		this(component, title, null);
	}
	
	public SimpleDialogPage(JComponent component, String title, String description) {
		this(component, title, description, null);
	}
	
	public SimpleDialogPage(JComponent component, String title, String description, Icon icon) {
		if (component == null) throw new IllegalArgumentException("component"); //$NON-NLS-1$
		this.component = component;
		setDescription(description);
		setTitle(title);
		setIcon(icon);
	}

	@Override
	protected JComponent createComponent() {
		return component;
	}

	public boolean isPageValid() {
		return true;
	}

}
