/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.dialogs;

import com.jidesoft.dialog.ButtonPanel;
import com.onaro.client.leekui.jface.preferences.Messages;
import com.onaro.client.leekui.jface.util.DialogUtils;
import com.onaro.client.leekui.ui.internal.Resources;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.jface.dialogs.TitleAreaDialog
 */
public class TitleAreaDialog extends JDialog implements IDialogPageContainer, IMessageConsumer {
	
	/**
	 * The title banner icon's size is 48x48. If a different size icon is specified,
	 * the icon will be scaled to this size.
	 */
	public static final Dimension BANNER_ICON_SIZE = new Dimension(48, 48);
	
	/**
	 * Standard return code constant (value 0) indicating that the window was
	 * opened.
	 * 
	 * @see #show(Window, String, IDialogPage)
	 */
	public static final int OK = 0;

	/**
	 * Standard return code constant (value 1) indicating that the window was
	 * canceled.
	 * 
	 * @see #show(Window, String, IDialogPage)
	 */
	public static final int CANCEL = 1;
	
	public static final Dimension MINUMUM_SIZE = new Dimension(400,250);

	private static final long serialVersionUID = 1L;
	private JPanel titleAreaPanel = null;
	private JPanel jContentPane = null;
	private ButtonPanel buttonPanel = null;
	private JButton affirmativeButton = null;
	private JButton cancelButton = null;
	private JComponent dialogArea = null;
	
	private JLabel titleLabel = null;
	private JLabel messageLabel = null;
	private JLabel imageLabel = null;
	
	private IDialogPage dialogPage;
	private IValidatableDialogPage validatablePage;
	
	private int returnCode = CANCEL;
	
	
	/**
	 * This method initializes 
	 * 
	 */
	public TitleAreaDialog() {
		super();
		initialize();
	}
	
	public TitleAreaDialog(Dialog dialog, IDialogPage dialogPage) {
		super(dialog);
		initialize();
		setDialogPage(dialogPage);
	}
	
	public TitleAreaDialog(Frame frame, IDialogPage dialogPage) {
		super(frame);
		initialize();
		setDialogPage(dialogPage);
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		this.setMinimumSize(MINUMUM_SIZE);
        this.setSize(MINUMUM_SIZE);
        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.setContentPane(getJContentPane());	
        this.addWindowListener(new java.awt.event.WindowAdapter() { 
        	public void windowClosing(WindowEvent e) {    
        		cancelPressed();
        	}
        });
        this.getRootPane().setDefaultButton(getAffirmativeButton());
	}

	/**
	 * This method initializes titleAreaPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	protected JPanel getTitleAreaPanel() {
		if (titleAreaPanel == null) {
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.insets = new Insets(5, 10, 5, 0);
			gridBagConstraints3.gridx = 1;
			gridBagConstraints3.gridy = 0;
			gridBagConstraints3.gridheight = 2;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 1;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.insets = new Insets(0, 20, 0, 0);
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints.gridwidth = 1;
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			gridBagConstraints.weightx = 0.0;
			gridBagConstraints.insets = new Insets(10, 10, 0, 0);
			titleAreaPanel = new JPanel();
			titleAreaPanel.setLayout(new GridBagLayout());
			titleAreaPanel.setBackground(Color.white);
			titleAreaPanel.setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.lightGray));
			titleAreaPanel.add(getTitleLabel(), gridBagConstraints);
			titleAreaPanel.add(getMessageLabel(), gridBagConstraints1);
			titleAreaPanel.add(getImageLabel(), gridBagConstraints3);
		}
		return titleAreaPanel;
	}

	/**
	 * This method initializes jContentPane	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getTitleAreaPanel(), BorderLayout.NORTH);
			jContentPane.add(getButtonPanel(), BorderLayout.SOUTH);
			jContentPane.add(getDialogArea(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes buttonPanel	
	 * 	
	 * @return com.jidesoft.dialog.ButtonPanel	
	 */
	protected ButtonPanel getButtonPanel() {
		if (buttonPanel == null) {
			buttonPanel = new ButtonPanel();
			buttonPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.lightGray), BorderFactory.createEmptyBorder(5, 5, 5, 5)));
			buttonPanel.setSizeContraint(ButtonPanel.SAME_SIZE);
			buttonPanel.setGroupGap(10);
			buttonPanel.add(getAffirmativeButton(), com.jidesoft.dialog.ButtonPanel.AFFIRMATIVE_BUTTON);
			buttonPanel.add(getCancelButton(), com.jidesoft.dialog.ButtonPanel.CANCEL_BUTTON);
		}
		return buttonPanel;
	}

	/**
	 * This method initializes affirmativeButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	protected JButton getAffirmativeButton() {
		if (affirmativeButton == null) {
			affirmativeButton = new JButton();
			affirmativeButton.setName("affirmativeButton"); //$NON-NLS-1$
			affirmativeButton.setText(Messages.INSTANCE.getOKButtonLabel());
			affirmativeButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					okPressed();
				}
			});
		}
		return affirmativeButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	protected JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
            cancelButton.setMnemonic(KeyEvent.VK_C);
            cancelButton.setName("cancelButton"); //$NON-NLS-1$
			cancelButton.setText(Messages.INSTANCE.getCancelButtonLabel());
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cancelPressed();
				}
			});
		}
		return cancelButton;
	}

	/**
	 * This method initializes dialogArea
	 * 	
	 * @return java.awt.Component
	 */
	protected Component getDialogArea() {
		if (dialogPage == null) {
			dialogArea = new JPanel();
		} else {
			dialogArea = dialogPage.getComponent(true);
		}
		return dialogArea;
	}
	
	/**
	 * This method initializes titleLabel	
	 * 	
	 * @return javax.swing.JLabel	
	 */
	private JLabel getTitleLabel() {
		if (titleLabel == null) {
			titleLabel = new JLabel();
			titleLabel.setFont(titleLabel.getFont().deriveFont(Font.BOLD, 14f));
			titleLabel.setText(Messages.INSTANCE.getTitleLabel());
		}
		return titleLabel;
	}

	/**
	 * This method initializes messageLabel	
	 * 	
	 * @return javax.swing.JLabel	
	 */
	private JLabel getMessageLabel() {
		if (messageLabel == null) {
			messageLabel = new JLabel();
			messageLabel.setFont(messageLabel.getFont().deriveFont(Font.PLAIN));
			messageLabel.setText(Messages.INSTANCE.getMessageLabel());
		}
		return messageLabel;
	}

	/**
	 * This method initializes imageLabel	
	 * 	
	 * @return javax.swing.JLabel	
	 */
	private JLabel getImageLabel() {
		if (imageLabel == null) {
			imageLabel = new JLabel();
			imageLabel.setPreferredSize(BANNER_ICON_SIZE);
			imageLabel.setText(""); //$NON-NLS-1$
		}
		return imageLabel;
	}
	
	public IDialogPage getDialogPage() {
		return dialogPage;
	}

	public void setDialogPage(IDialogPage dialogPage) {
		if (this.dialogPage != dialogPage) {
			if (this.validatablePage != null) {
				this.validatablePage.setContainer(null);
			}
			
			this.dialogPage = dialogPage;
			if (this.dialogPage instanceof IValidatableDialogPage) {
				this.validatablePage = (IValidatableDialogPage)this.dialogPage;
				this.validatablePage.setContainer(this);
			} else {
				this.validatablePage = null;
			}
			
			updateTitle();
			updateMessage();
			updateButtons();
			updateDialogArea();
		}
	}

	public String getMessage() {
		return getMessageLabel().getText();
	}
	
	public void setMessage(String message) {
		setMessage(message, IMessageProvider.Type.NONE);
	}
	
	public void setMessage(String message, IMessageProvider.Type type) {
		Icon newIcon;
		
		switch (type) {
			case ERROR : newIcon = Resources.INSTANCE.getErrorIcon(); break;
			case WARNING : newIcon = Resources.INSTANCE.getWarningIcon(); break;
			case INFO : newIcon = Resources.INSTANCE.getInfoIcon(); break;
			case NONE :
			default: newIcon = null;
		}
		
		getMessageLabel().setText(message);
		getMessageLabel().setIcon(newIcon);
	}
	
	public String getTitleText() {
		return getTitleLabel().getText();
	}
	
	public void setTitleText(String title) {		
		getTitleLabel().setText(title);
	}
	
	public Icon getTitleIcon() {
		return getImageLabel().getIcon();
	}
	
	public void setTitleIcon(Icon icon) {
		if (icon == null) {
			getImageLabel().setIcon(null);
			return;
		}
		
		ImageIcon imageIcon = (ImageIcon)icon;
		Image img = imageIcon.getImage();
		Dimension size = getImageLabel().getPreferredSize();
		Image result = img.getScaledInstance(size.width, size.height, Image.SCALE_SMOOTH);
		imageIcon.setImage(result);
		getImageLabel().setIcon(imageIcon);
	}
	
	public void updateButtons() {
		if (validatablePage != null) {
			boolean valid = validatablePage.isPageValid();
			getAffirmativeButton().setEnabled(valid);
		}
	}

	public void updateMessage() {
		if (dialogPage != null) {
			DialogUtils.consumeMessage(this, dialogPage);
		}
	}

	public void updateTitle() {
		if (dialogPage != null) {
			setTitleText(dialogPage.getTitle());
			Icon icon = dialogPage.getIcon();
			if (icon != null) {
				setTitleIcon(icon);
			}
		}
	}
	
	private void updateDialogArea() {
		if (dialogArea != null) {
			getJContentPane().remove(dialogArea);
		}
		Component newDialogArea = getDialogArea();
		getJContentPane().add(newDialogArea, BorderLayout.CENTER);
	}
	
	/**
	 * Notifies that the ok button of this dialog has been pressed.
	 * <p>
	 * The <code>Dialog</code> implementation of this framework method sets
	 * this dialog's return code to <code>Window.OK</code> and closes the
	 * dialog. Subclasses may override.
	 * </p>
	 */
	protected void okPressed() {
		if (validatablePage != null) {
			if (!validatablePage.performOk()) {
				return;
			}
		}
		setReturnCode(OK);
		close();
	}
	
	/**
	 * Notifies that the ok button of this dialog has been pressed.
	 * <p>
	 * The <code>Dialog</code> implementation of this framework method sets
	 * this dialog's return code to <code>Window.OK</code> and closes the
	 * dialog. Subclasses may override.
	 * </p>
	 */
	protected void cancelPressed() {
		if (validatablePage != null) {
			if (!validatablePage.performCancel()) {
				return;
			}
		}
		setReturnCode(CANCEL);
		close();
	}
	
	public void close() {
		setVisible(false);
	}
	
	protected void setReturnCode(int code) {
		this.returnCode = code;
	}
	
	public int getReturnCode() {
		return returnCode;
	}
	
	public static int show(Window parent, String dialogTitle, IDialogPage dialogPage) {
		return show(parent, dialogTitle, dialogPage, true);
	}
	
	public static int show(Window parent, String dialogTitle, IDialogPage dialogPage, boolean modal) {
		TitleAreaDialog dialog;
		if (parent instanceof Dialog) {
			dialog = new TitleAreaDialog((Dialog)parent, dialogPage);
		} else if (parent instanceof Frame) {
			dialog = new TitleAreaDialog((Frame)parent, dialogPage);
		} else {
			dialog = new TitleAreaDialog((Frame)null, dialogPage);
		}
		
		dialog.setTitle(dialogTitle);
		dialog.setName(dialogTitle);
		dialog.setModal(modal);
		dialog.pack();
		
		Dimension size = dialog.getSize();
		size.height = Math.max(size.height, MINUMUM_SIZE.height);
		size.width = Math.max(size.width, MINUMUM_SIZE.width);
		dialog.setSize(size);
		
		dialog.setLocationRelativeTo(parent);
		dialog.setVisible(true);
		
		return dialog.getReturnCode();
	}


}  //  @jve:decl-index=0:visual-constraint="10,10"
