/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.util;

import com.onaro.client.leekui.jface.dialogs.DialogPage;
import com.onaro.client.leekui.jface.dialogs.IDialogPage;
import com.onaro.client.leekui.jface.dialogs.IDialogPageContainer;
import com.onaro.client.leekui.jface.dialogs.IMessageConsumer;
import com.onaro.client.leekui.jface.dialogs.IMessageProvider;
import com.onaro.client.leekui.jface.dialogs.ValidatableDialogPage;

public class DialogUtils {
	
	public static void dispatchMessage(DialogPage dialogPage, String message, IMessageProvider.Type messageType) {		
		switch (messageType) {
			case ERROR : dialogPage.setErrorMessage(message); break;
			case WARNING : dialogPage.setWarningMessage(message); break;
			case INFO : dialogPage.setMessage(message); break;
			case NONE : dialogPage.setDescription(message); break;
			default : break;
		}
		
		if (dialogPage instanceof ValidatableDialogPage) {
			IDialogPageContainer container = ((ValidatableDialogPage)dialogPage).getContainer();
			if (container != null) {
				container.updateMessage();
			}
		}
	}
	
	public static void consumeMessage(IMessageConsumer consumer, IDialogPage provider) {
		if (provider.getErrorMessage() != null) {
			consumer.setMessage(provider.getErrorMessage(), IMessageProvider.Type.ERROR);
		} else if (provider.getWarningMessage() != null) {
			consumer.setMessage(provider.getWarningMessage(), IMessageProvider.Type.WARNING);
		} else if (provider.getMessage() != null) {
			consumer.setMessage(provider.getMessage(), IMessageProvider.Type.INFO);
		} else {
			consumer.setMessage(provider.getDescription(), IMessageProvider.Type.NONE);
		}
	}

}
