/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.util;

import org.eclipse.core.runtime.ISafeRunnable;

/**
 * Runs a safe runnables.
 * <p>
 * Clients may provide their own implementation to change
 * how safe runnables are run from within JFace.
 * </p>
 * 
 * @see SafeRunnable#getRunner()
 * @see SafeRunnable#setRunner(ISafeRunnableRunner)
 * @see SafeRunnable#run(ISafeRunnable)
 * 
 * This class is a reimplementation of the Eclipse class org.eclipse.jface.util.ISafeRunnableRunner
 */
public interface ISafeRunnableRunner {
	
	/**
	 * Runs the runnable.  All <code>ISafeRunnableRunners</code> must catch any exception
	 * thrown by the <code>ISafeRunnable</code> and pass the exception to 
	 * <code>ISafeRunnable.handleException()</code>.
	 *
	 * @see SafeRunnable#run(ISafeRunnable)
	 */
	public abstract void run(ISafeRunnable code);
	
}
