/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.viewers;

import javax.swing.event.EventListenerList;

import com.onaro.client.leekui.jface.util.SafeRunnable;

public abstract class AbstractSelectionProvider implements ISelectionProvider 
{
	/**
	 * List of selection change listeners (element type: <code>ISelectionChangedListener</code>).
	 * 
	 * @see #fireSelectionChanged
	 */
	private EventListenerList selectionChangedListeners;

	/**
	 * Creates a new viewer.
	 */
	protected AbstractSelectionProvider() {
		selectionChangedListeners = new EventListenerList();
	}

	/*
	 * (non-Javadoc) Method declared on ISelectionProvider.
	 */
	public void addSelectionChangedListener(ISelectionChangedListener listener) {
		selectionChangedListeners.add(ISelectionChangedListener.class, listener);
	}

	/*
	 * (non-Javadoc) Method declared on ISelectionProvider.
	 */
	public void removeSelectionChangedListener(ISelectionChangedListener listener) {
		selectionChangedListeners.remove(ISelectionChangedListener.class, listener);
	}
	
	protected void fireSelectionChanged(ISelection selection)
	{
		SelectionChangedEvent event = new SelectionChangedEvent(this, selection);
		
		fireSelectionChanged(event);
	}

	/**
	 * Notifies any selection changed listeners that the viewer's selection has changed. Only listeners registered at
	 * the time this method is called are notified.
	 * 
	 * @param event
	 *            a selection changed event
	 * 
	 * @see ISelectionChangedListener#selectionChanged
	 */
	protected void fireSelectionChanged(final SelectionChangedEvent event) {
		
		Object[] listeners = selectionChangedListeners.getListeners(ISelectionChangedListener.class);
		for (int i = 0; i < listeners.length; ++i) {
			final ISelectionChangedListener l = (ISelectionChangedListener) listeners[i];
			SafeRunnable.run(new SafeRunnable() {
				public void run() {
					l.selectionChanged(event);
				}
			});
		}
	}
}
