/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui;

import java.awt.Image;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.IEditorInput
 */
public interface IEditorInput
{
	/**
	 * Returns whether the editor input exists.
	 * <p>
	 * This method is primarily used to determine if an editor input should
	 * appear in the "File Most Recently Used" menu. An editor input will appear
	 * in the list until the return value of <code>exists</code> becomes
	 * <code>false</code> or it drops off the bottom of the list.
	 * 
	 * @return <code>true</code> if the editor input exists;
	 *         <code>false</code> otherwise
	 */
	public boolean exists();
	
	public Image getImage();
	
	/**
	 * Returns the name of this editor input for display purposes.
	 * <p>
	 * For instance, when the input is from a file, the return value would
	 * ordinarily be just the file name.
	 * 
	 * @return the name string; never <code>null</code>;
	 */
	public String getName();
	
	public String getToolTipText();
	
	/**
	 * Returns an object that can be used to save the state of this editor
	 * input.
	 * 
	 * @return the persistable element, or <code>null</code> if this editor
	 *         input cannot be persisted
	 */
	public IPersistableElement getPersistable();
}
