/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui;

import java.util.*;


/**
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.IWorkbenchPage
 */
public interface IWorkbenchPage extends IPartService, ISelectionService {

	public static enum ViewMode {
		ACTIVATE, VISIBLE, CREATE
	};
	
	/**
	 * Editor opening match mode specifying that no matching against existing
	 * editors should be done.
	 * 
	 * 
	 */
	public static final int MATCH_NONE = 0;

	/**
	 * Editor opening match mode specifying that the editor input should be
	 * considered when matching against existing editors.
	 * 
	 * 
	 */
	public static final int MATCH_INPUT = 1;

	/**
	 * Editor opening match mode specifying that the editor id should be
	 * considered when matching against existing editors.
	 * 
	 * 
	 */
	public static final int MATCH_ID = 2;

	public IWorkbenchWindow getWorkbenchWindow();
	
	public IEditorPart openEditor(IEditorInput input, String editorId) throws PartInitException;

	public IEditorPart openEditor(IEditorInput input, String editorId, boolean activate) throws PartInitException;

	public IViewPart showView(String viewId) throws PartInitException;

	public IViewPart showView(String viewId, String secondaryId, ViewMode viewMode) throws PartInitException;

	public IEditorPart openEditor(final IEditorInput input, final String editorId, final boolean activate,
			final int matchFlags) throws PartInitException;
	
	public void setPerspective(IPerspectiveDescriptor perspective);
	
	/**
	 * Activates the given part. The part will be brought to the front and given
	 * focus. The part must belong to this page.
	 * 
	 * @param part
	 *            the part to activate
	 */
	public void activate(IWorkbenchPart part);
	
	/**
	 * Returns the current perspective descriptor for this page, or
	 * <code>null</code> if there is no current perspective.
	 */
	public IPerspectiveDescriptor getPerspective();
	
	/**
	 * Moves the given part forward in the Z order of this page so as to make it
	 * visible, without changing which part has focus. The part must belong to
	 * this page.
	 * 
	 * @param part
	 *            the part to bring forward
	 */
	public void bringToTop(IWorkbenchPart part);

	/**
	 * Returns a list of the reference to views on this page.
	 * <p>
	 * Note that each page has its own views; views are never shared between
	 * pages.
	 * </p>
	 */
	public List<IViewReference> getViewReferences();
	
	/**
	 * Changes the visible views, their layout, and the visible action sets
	 * within the page to match the current perspective descriptor. This is a
	 * rearrangement of components and not a replacement. The contents of the
	 * current perspective descriptor are unaffected.
	 * <p>
	 * For more information on perspective change see
	 * <code>setPerspective()</code>.
	 * </p>
	 */
	public void resetPerspective();

	
}
