/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui;

import java.beans.PropertyChangeListener;

import javax.swing.Icon;
import javax.swing.JComponent;

import org.eclipse.core.runtime.IAdaptable;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.IWorkbenchPart
 */
public interface IWorkbenchPart extends IAdaptable {

	public void addPropertyChangeListener(PropertyChangeListener pcListener);

	public void removePropertyChangeListener(PropertyChangeListener pcListener);

	/**
	 * Creates the component for this workbench part.
	 * <p>
	 * Clients should not call this method (the workbench calls this method when it needs to, which may be never).
	 * </p>
	 * <p>
	 * For implementors this is a multi-step process:
	 * <ol>
	 * <li>Create the component.</li>
	 * <li>Register any global actions with the site's <code>IActionBars</code>.</li>
	 * <li>Register any context menus with the site.</li>
	 * <li>Register a selection provider with the site, to make it available to the workbench's
	 * <code>ISelectionService</code> (optional). </li>
	 * </ol>
	 * </p>
	 * 
	 */
	public JComponent createPartComponent();

	/**
	 * Returns the title of this workbench part. If this value changes the part must fire a property listener event with
	 * property name <code>title</code>.
	 * <p>
	 * The title is used to populate the title bar of this part's visual container.
	 * </p>
	 * 
	 * @return the workbench part title (not <code>null</code>)
	 */
	public String getTitle();

	/**
	 * Returns the title image of this workbench part. If this value changes the part must fire a property listener
	 * event with property name <code>titleIcon</code>.
	 * <p>
	 * The property name differs from Eclipse.
	 * </p>
	 * <p>
	 * The title icon is usually used to populate the title bar of this part's visual container.
	 * </p>
	 * 
	 * @return the title image
	 */
	public Icon getTitleIcon();

	/**
	 * Returns the title tool tip text of this workbench part. An empty string result indicates no tool tip. If this
	 * value changes the part must fire a property listener event with property name <code>titleToolTip</code>.
	 * <p>
	 * The property name differs from Eclipse.
	 * </p>
	 * <p>
	 * The tool tip text is used to populate the title bar of this part's visual container.
	 * </p>
	 * 
	 * @return the workbench part title tool tip (not <code>null</code>)
	 */
	public String getTitleToolTip();

	/**
	 * Returns the name of this part. If this value changes the part must fire a property listener event with property
	 * name <code>partName</code>.
	 * 
	 * @return the name of this view, or the empty string if the name is being managed by the workbench (not
	 *         <code>null</code>)
	 */
	public String getPartName();

	/**
	 * Returns the content description of this part. The content description is an optional user-readable string that
	 * describes what is currently being displayed in the part. By default, the workbench will display the content
	 * description in a line near the top of the view or editor. An empty string indicates no content description text.
	 * If this value changes the part must fire a property listener event with property name
	 * <code>contentDescriptor</code>.
	 * 
	 * @return the content description of this part (not <code>null</code>)
	 */
	public String getContentDescription();

	/**
	 * Disposes of this workbench part.
	 * <p>
	 * This is the last method called on the <code>IWorkbenchPart</code>. There is no guarantee that
	 * createPartControl() has been called, so the part components may never have been created.
	 * </p>
	 * <p>
	 * Within this method a part may release any resources, etc.&nbsp; held by this part. It is also very important to
	 * deregister all listeners from the workbench.
	 * </p>
	 * <p>
	 * Clients should not call this method (the workbench calls this method at appropriate times).
	 * </p>
	 */
	public void dispose();

	/**
	 * Asks this part to take focus within the workbench.
	 * <p>
	 * Clients should not call this method (the workbench calls this method at appropriate times). To have the workbench
	 * activate a part, use <code>IWorkbenchPage.activate(IWorkbenchPart) instead</code>.
	 * </p>
	 */
	public void setFocus();

	/**
	 * Returns the site for this workbench part. The site can be <code>null</code> while the workbench part is being
	 * initialized. After the initialization is complete, this value must be non-<code>null</code> for the remainder
	 * of the part's life cycle.
	 * 
	 * @return The part site; this value may be <code>null</code> if the part has not yet been initialized
	 */
	public IWorkbenchPartSite getSite();
}
