/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui;

import java.util.EventObject;

public class PartEvent extends EventObject {

	private static final long serialVersionUID = 1L;

	public static enum Type {
		/**
		 * Notifies this listener that the given part has been activated.
		 */
		ACTIVATED,

		/**
		 * Notifies this listener that the given part has been brought to the top.
		 * <p>
		 * These events occur when an editor is brought to the top in the editor area, or when a view is brought to the
		 * top in a page book with multiple views. They are normally only sent when a part is brought to the top
		 * programmatically (via <code>IPerspective.bringToTop</code>). When a part is activated by the user clicking
		 * on it, only <code>partActivated</code> is sent.
		 * </p>
		 */
		BROUGHT_TO_TOP, 
		
		/**
	     * Notifies this listener that the given part has been closed.
	     */
		CLOSED, 
		
		/**
	     * Notifies this listener that the given part has been deactivated.
	     */
		DEACTIVATED, 
		
		/**
	     * Notifies this listener that the given part has been opened.
	     */
		HIDDEN, INPUT_CHANGED, OPENED, VISIBLE
	};

	private Type type;

	private IWorkbenchPartReference partReference;

	public PartEvent(IWorkbenchWindow window, Type type, IWorkbenchPartReference partRef) {
		super(window);

		this.type = type;
		this.partReference = partRef;
	}

	public IWorkbenchWindow getWindow() {
		return (IWorkbenchWindow) getSource();
	}

	public IWorkbenchPartReference getPartReference() {
		return partReference;
	}

	public Type getType() {
		return type;
	}

}
