/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.compatibility;

import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JOptionPane;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.osgi.util.NLS;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import com.jidesoft.status.StatusBarItem;
import com.onaro.client.leekui.jface.preferences.IPreferenceNode;
import com.onaro.client.leekui.jface.preferences.IPreferencePage;
import com.onaro.client.leekui.jface.preferences.PreferenceNode;
import com.onaro.client.leekui.ui.IEditorDescriptor;
import com.onaro.client.leekui.ui.IEditorMatchingStrategy;
import com.onaro.client.leekui.ui.IEditorPart;
import com.onaro.client.leekui.ui.IMemento;
import com.onaro.client.leekui.ui.IPerspectiveDescriptor;
import com.onaro.client.leekui.ui.IPerspectiveFactory;
import com.onaro.client.leekui.ui.IViewPart;
import com.onaro.client.leekui.ui.IWorkbenchPage;
import com.onaro.client.leekui.ui.PlatformUI;
import com.onaro.client.leekui.ui.XMLMemento;
import com.onaro.client.leekui.ui.internal.IWorkbenchConstants;
import com.onaro.client.leekui.ui.internal.ReferenceCounter;
import com.onaro.client.leekui.ui.internal.Workbench;
import com.onaro.client.leekui.ui.internal.WorkbenchMessages;
import com.onaro.client.leekui.ui.internal.WorkbenchPage;
import com.onaro.client.leekui.ui.internal.WorkbenchPlugin;
import com.onaro.client.leekui.ui.internal.compatibility.EditorDescriptor;
import com.onaro.client.leekui.ui.internal.compatibility.PerspectiveDescriptor;
import com.onaro.client.leekui.ui.internal.compatibility.ViewDescriptor;
import com.onaro.client.leekui.ui.internal.dialogs.ShowViewDialogContent;
import com.onaro.client.leekui.ui.internal.progress.ProgressManager;
import com.onaro.client.leekui.ui.views.IViewDescriptor;

/**
 * This class provides compatability between the 3.7 to 4.0 release.
 * 
 * In the future this functionality will be replaced by extension points.
 */
@Deprecated
public final class CompatibilityUtils {

	static final Logger logger = Logger.getLogger(CompatibilityUtils.class);
	
	private CompatibilityUtils() {
		// do nothing
	}

	/**
	 * @see #registerPerspectiveClass(Class, String, String, boolean)
	 */
	public static IPerspectiveDescriptor registerPerspectiveClass(Class<? extends IPerspectiveFactory> clazz,
			String id, String label) {
		return registerPerspectiveClass(clazz, id, label, false);
	}

	public static IPerspectiveDescriptor registerPerspectiveClass(Class<? extends IPerspectiveFactory> clazz,
			String id, String label, boolean fixed) {
		if (clazz == null) throw new IllegalArgumentException();

		id = safeId(clazz, id);
		label = safeLabel(clazz, label);

		PerspectiveDescriptor perspectiveDescriptor = new PerspectiveDescriptor(clazz, id, label, fixed);

		Workbench.getInstance().getPerspectiveRegistry().add(perspectiveDescriptor);

		return perspectiveDescriptor;
	}

	/**
	 * @see #registerViewClass(Class,String,String,Icon,boolean,String)
	 */
	public static IViewDescriptor registerViewClass(Class<? extends IViewPart> clazz) {
		return registerViewClass(clazz, null);
	}

	/**
	 * @see #registerViewClass(Class,String,String,Icon,boolean,String)
	 */
	public static IViewDescriptor registerViewClass(Class<? extends IViewPart> clazz, String id) {
		return registerViewClass(clazz, id, null);
	}

	/**
	 * @see #registerEditorClass(Class, String, String, IEditorMatchingStrategy)
	 */
	public static IEditorDescriptor registerEditorClass(Class<? extends IEditorPart> clazz) {
		return registerEditorClass(clazz, null);
	}

	/**
	 * @see #registerEditorClass(Class, String, String, IEditorMatchingStrategy)
	 */
	public static IEditorDescriptor registerEditorClass(Class<? extends IEditorPart> clazz, String id) {
		return registerEditorClass(clazz, id, null);
	}

	/**
	 * @see #registerEditorClass(Class, String, String, IEditorMatchingStrategy)
	 */
	public static IEditorDescriptor registerEditorClass(Class<? extends IEditorPart> clazz, String id, String label) {
		return registerEditorClass(clazz, id, label, null);
	}

	/**
	 * 
	 * @param clazz
	 * @param id
	 *            a unique name that will be used to identify this view, if <code>null</code> then id is the editor
	 *            class's name.
	 * @param label
	 *            a translatable name that will be used in the UI for this view.
	 * @param editorMatchingStrategy
	 */
	public static IEditorDescriptor registerEditorClass(Class<? extends IEditorPart> clazz, String id, String label,
			IEditorMatchingStrategy editorMatchingStrategy) {
		if (clazz == null) throw new IllegalArgumentException();

		id = safeId(clazz, id);
		label = safeLabel(clazz, label);

		IEditorDescriptor editorDescriptor = new EditorDescriptor(clazz, id, label, editorMatchingStrategy);

		Workbench.getInstance().getEditorRegistry().add(editorDescriptor);

		return editorDescriptor;
	}

	/**
	 * @see #registerViewClass(Class,String,String,Icon,boolean,String)
	 */
	public static IViewDescriptor registerViewClass(Class<? extends IViewPart> clazz, String id, String label) {
		return registerViewClass(clazz, id, label, null);
	}

	/**
	 * @see #registerViewClass(Class,String,String,Icon,boolean,String)
	 */
	public static IViewDescriptor registerViewClass(Class<? extends IViewPart> clazz, String id, String label, Icon icon) {
		return registerViewClass(clazz, id, label, icon, false);
	}

	/**
	 * @see #registerViewClass(Class,String,String,Icon,boolean,String)
	 */
	public static IViewDescriptor registerViewClass(Class<? extends IViewPart> clazz, String id, String label,
			Icon icon, boolean allowMultiple) {
		return registerViewClass(clazz, id, label, icon, allowMultiple, null);
	}

	/**
	 * 
	 * @param clazz
	 * @param id
	 *            a unique name that will be used to identify this view, if <code>null</code> then id is the view
	 *            class's name.
	 * @param label
	 *            a translatable name that will be used in the UI for this view.
	 * @param icon
	 * @param allowMultiple
	 *            flag indicating whether this view allows multiple instances to be created using
	 *            IWorkbenchPage.showView(String id, String secondaryId). The default is false.
	 * @param category
	 *            an optional attribute that is composed of the category IDs separatedby '/'. Each referenced category
	 *            must be declared in a corresponding category element.
	 */
	public static IViewDescriptor registerViewClass(Class<? extends IViewPart> clazz, String id, String label,
			Icon icon, boolean allowMultiple, String category) {
		if (clazz == null) throw new IllegalArgumentException();

		id = safeId(clazz, id);
		label = safeLabel(clazz, label);

		List<String> categoryPath = null;

		if (category != null && category.length() > 0) {
			categoryPath = Arrays.asList(category.split("/")); //$NON-NLS-1$
		}

		ViewDescriptor viewDescriptor = new ViewDescriptor(clazz, id, label, icon, allowMultiple, categoryPath);

		Workbench.getInstance().getViewRegistry().add(viewDescriptor);

		return viewDescriptor;
	}
	
	public static IPreferenceNode registerPreferencePageClass(Class<? extends IPreferencePage> clazz, String id, String label) {
		return registerPreferencePageClass(clazz, id, label, "/"); //$NON-NLS-1$
	}
	
	public static IPreferenceNode registerPreferencePageClass(Class<? extends IPreferencePage> clazz, String id, String label, String path) {
		IPreferenceNode preferenceNode = new PreferenceNode(id, label, clazz);
		
		Workbench.getInstance().getPreferenceManager().addTo(path, preferenceNode);
		
		return preferenceNode;
	}

	public static Action createResetPerspectiveAction(IWorkbenchPage page) {
		WorkbenchPage pageImpl = (WorkbenchPage) page;

		return new ResetPerspectiveAction(pageImpl);
	}

	/**
	 * Create the action used to launch the show view dialog.
	 */
	public static Action createShowViewDialogAction(IWorkbenchPage page) {
		return createShowViewDialogAction(page, null);
	}
	
	/**
	 * @param categoryId (Optional) only those views under this category. 
	 */
	public static Action createShowViewDialogAction(IWorkbenchPage page, String categoryId) {
		WorkbenchPage pageImpl = (WorkbenchPage) page;

		ShowViewDialogAction action = new ShowViewDialogAction();
		action.setPage(pageImpl);
		action.setCategoryId(categoryId);
		
		return action;
	}

	static class ResetPerspectiveAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		private WorkbenchPage page;

		public ResetPerspectiveAction(WorkbenchPage page) {
			if (page == null) throw new IllegalArgumentException();

			this.page = page;
			
			putValue(Action.NAME, WorkbenchMessages.ResetPerspective_Item);
			putValue(Action.SHORT_DESCRIPTION, WorkbenchMessages.ResetPerspective_toolTip);
		}

		public void actionPerformed(ActionEvent event) {
			
			IPerspectiveDescriptor persp = page.getPerspective();
			
			String message = NLS.bind(WorkbenchMessages.ResetPerspective_message, persp.getLabel() );
			
			int result = JOptionPane.showConfirmDialog(page.getComponent(), message, null, JOptionPane.OK_CANCEL_OPTION);
			
			if (result != JOptionPane.OK_OPTION) return;
			
			page.resetPerspective();
			
		}
	}

	static class ShowViewDialogAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		private WorkbenchPage page;
		
		private String categoryId;

		public ShowViewDialogAction() {
			//do nothing
		}
		
		public void actionPerformed(ActionEvent event) {
			
			Component parent = null; 
			
			if (page != null)
			{
				parent = page.getComponent();
			}

			ShowViewDialogContent dialogContent = new ShowViewDialogContent(categoryId);

			String title = WorkbenchMessages.ShowView_Title;
			
			JOptionPane optionPane = new JOptionPane(dialogContent, JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
			JDialog dialog = optionPane.createDialog(parent, title);
			dialog.setResizable(true);
			dialog.setVisible(true);

			Object value = optionPane.getValue();

			if (value == null || !value.equals(JOptionPane.OK_OPTION)) return;
			
			for (IViewDescriptor viewDescriptor : dialogContent.getSelectedViewDescriptors()) {
				try {
					getPage().showView(viewDescriptor.getId(), null, IWorkbenchPage.ViewMode.ACTIVATE);
				}
				catch (Throwable t) {
					WorkbenchPlugin.log(t);
				}
			}
		}

		public final WorkbenchPage getPage() {
			return page;
		}

		public final void setPage(WorkbenchPage page) {
			this.page = page;
		}

		public final String getCategoryId() {
			return categoryId;
		}

		public final void setCategoryId(String categoryId) {
			this.categoryId = categoryId;
		}
	}

	/**
	 * @see #createViewCategory(String, String, String)
	 */
	public static void createViewCategory(String id, String name) {
		createViewCategory(id, name, null);
	}

	/**
	 * @param id
	 *            a unique name that will be used to identify this category.
	 * @param name
	 *            a translatable name that will be used in the UI for this category.
	 * @param parentCategory
	 *            an optional path composed of category IDs separated by '/'.
	 */
	public static void createViewCategory(String id, String name, String parentCategory) {
		Workbench.getInstance().getViewRegistry().createCategory(id, name, parentCategory);
	}

	public static Container getPageContainer(IWorkbenchPage workbenchPage) {
		return ((WorkbenchPage) workbenchPage).getComponent();
	}
	
	public static void showProgressDialog() {
		ProgressManager.getInstance().showProgressDialog();
	}
		
	public static StatusBarItem getProgressStatusBarItem() {
		return ProgressManager.getInstance().getProgressStatusBarItem();
	}

	public static IWorkbenchPage createWorkbenchPage() {
		return new WorkbenchPage();
	}

	public static IStatus saveState(IWorkbenchPage page, Preferences preferences) {

		if (page == null) throw new IllegalArgumentException();
		if (preferences == null) throw new IllegalArgumentException();

		IStatus status = null;

		try {

			XMLMemento memento = XMLMemento.createWriteRoot(IWorkbenchConstants.TAG_PAGE);

			status = saveState(page, memento);

			exportMemento(memento.getElement(), preferences);
		}
		catch (Exception e) {
			status = new Status(Status.ERROR, PlatformUI.PLUGIN_ID, 0, "", e); //$NON-NLS-1$
		}

		return status;
	}

	public static IStatus restoreState(IWorkbenchPage page, Preferences preferences,
			IPerspectiveDescriptor activePerspective) {
		IStatus status = null;

		if (page == null) throw new IllegalArgumentException();
		if (preferences == null) throw new IllegalArgumentException();
		
		try {
			XMLMemento memento = XMLMemento.createWriteRoot(IWorkbenchConstants.TAG_PAGE);

			importMemento(memento.getElement(), preferences);

			status = restoreState(page, memento, activePerspective);
		}
		catch (Exception e) {
			status = new Status(Status.ERROR, PlatformUI.PLUGIN_ID, 0, "", e); //$NON-NLS-1$
		}

		return status;
	}

	public static IStatus restoreState(IWorkbenchPage page, IMemento memento, IPerspectiveDescriptor activePerspective) {
		WorkbenchPage pageImpl = (WorkbenchPage) page;

		IStatus status = pageImpl.restoreState(memento, activePerspective);

		return status;
	}

	public static IStatus saveState(IWorkbenchPage page, IMemento memento) {
		WorkbenchPage pageImpl = (WorkbenchPage) page;

		return pageImpl.saveState(memento);
	}

	private static String safeId(Class<?> clazz, String id) {
		if (id != null && id.length() > 0) return id;

		return clazz.getName();
	}

	private static String safeLabel(Class<?> clazz, String label) {
		if (label != null && label.length() > 0) return label;

		return clazz.getSimpleName();
	}

	/**
	 * The output of this is persisted across versions and the structure should not be changed.
	 */
	private static void exportMemento(Element xmlMementoElement, Preferences preferences) {
		NamedNodeMap attributes = xmlMementoElement.getAttributes();

		for (int i = 0; i < attributes.getLength(); i++) {
			Attr attr = (Attr) attributes.item(i);

			String attrValue = attr.getValue();

			if (attrValue == null || attrValue.length() < 1) continue;

			try
			{
				preferences.put("attr_" + attr.getName(), attrValue); //$NON-NLS-1$
			}
			catch (Exception e)
			{
				logger.warn(e.getMessage(), e);
			}
		}

		NodeList childNodes = xmlMementoElement.getChildNodes();

		if (childNodes.getLength() > 0) {

			ReferenceCounter<String,Object> refCounter = new ReferenceCounter<String,Object>();

			for (int i = 0; i < childNodes.getLength(); i++) {
				Node childNode = childNodes.item(i);

				if (childNode instanceof Element) {
					final String nodeName = childNode.getNodeName();

					if (refCounter.get(nodeName) == null) {
						refCounter.put(nodeName, new Object());
					}

					int refCount = refCounter.getRef(nodeName);

					StringBuilder nodePath = new StringBuilder(nodeName);

					nodePath.append("_"); //$NON-NLS-1$
					nodePath.append(refCount);

					Preferences childPrefs = preferences.node(nodePath.toString());

					exportMemento((Element) childNode, childPrefs);

					refCounter.addRef(nodeName);
				}
				else if (childNode instanceof Text) {
					String nodeValue = childNode.getNodeValue();

					if (nodeValue != null && nodeValue.length() > 0) {
						List<String> chunks = splitBySize(nodeValue, Preferences.MAX_VALUE_LENGTH);

						int numChunks = 0;
						for (String chunk : chunks) {
							preferences.put("text_chunk_" + (numChunks++), chunk); //$NON-NLS-1$
						}

						preferences.putInt("text_num_chunks", numChunks); //$NON-NLS-1$
					}
				}
				else {
					String msg = "Failed to export child node of type: " + childNode.getNodeType(); //$NON-NLS-1$
					throw new RuntimeException(msg);
				}
			}
		}
	}

	/**
	 * Split a string into chunks of a maximum length.
	 */
	private static List<String> splitBySize(String s, int chunkSize) {
		if (s == null) throw new IllegalArgumentException();
		if (chunkSize < 1) throw new IllegalArgumentException();

		List<String> chunks = new ArrayList<String>();

		for (int beginIndex = 0; beginIndex < s.length(); beginIndex += chunkSize) {
			int endIndex = Math.min(beginIndex + chunkSize, s.length());

			String chunk = s.substring(beginIndex, endIndex);
			chunks.add(chunk);
		}

		return chunks;
	}

	private static void importMemento(Element memento, Preferences preferences) throws BackingStoreException {
		Set<String> keys = new HashSet<String>();

		Collections.addAll(keys, preferences.keys());

		// import attrs
		for (String key : keys) {
			if (!key.startsWith("attr_")) continue; //$NON-NLS-1$

			String value = preferences.get(key, ""); //$NON-NLS-1$

			String name = key.substring("attr_".length()); //$NON-NLS-1$

			memento.setAttribute(name, value);
		}

		// import text
		{
			int numChunks = preferences.getInt("text_num_chunks", 0); //$NON-NLS-1$

			if (numChunks > 0) {
				String[] chunks = new String[numChunks];

				for (int i = 0; i < numChunks; i++) {
					chunks[i] = preferences.get("text_chunk_" + i, ""); //$NON-NLS-1$ //$NON-NLS-2$
				}

				String textContent = StringUtils.join(chunks);

				memento.setTextContent(textContent);
			}
		}

		for (String childName : preferences.childrenNames()) {
			Preferences childPrefs = preferences.node(childName);

			childName = childName.substring(0, childName.lastIndexOf('_'));

			Element childElement = memento.getOwnerDocument().createElement(childName);
			memento.appendChild(childElement);

			importMemento(childElement, childPrefs);
		}
	}

	/**
	 * For debugging / development.
	 */
	public static void main(String[] args) throws Exception
	{
		ShowViewDialogAction action = new ShowViewDialogAction();
		action.actionPerformed(null);
	}
}
