/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.part;

import com.onaro.client.leekui.ui.IEditorInput;
import com.onaro.client.leekui.ui.IEditorPart;
import com.onaro.client.leekui.ui.IEditorSite;
import com.onaro.client.leekui.ui.IWorkbenchPartSite;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.part.EditorPart
 */
public abstract class EditorPart extends WorkbenchPart implements IEditorPart
{
	private IEditorInput editorInput = null;
	
	public IEditorSite getEditorSite() {
        return (IEditorSite) getSite();
    }
	
	@Override
	public final String getTitleToolTip() {
        if (editorInput == null) {
			return super.getTitleToolTip();
		} 
	
        return editorInput.getToolTipText();
    }
	
	public IEditorInput getEditorInput() {
        return editorInput;
    }
	
	protected void setInput(IEditorInput input) {
        editorInput = input;
    }
	
	protected final void setInputWithNotify(IEditorInput input) {
		
		IEditorInput oldInput = this.editorInput;
		
		setInput(input);
        
        this.firePropertyChange("editorInput", oldInput, input); //$NON-NLS-1$
    }
	
	@Override
	protected final void checkSite(IWorkbenchPartSite site) {
        super.checkSite(site);
        
        if (!IEditorSite.class.isInstance(site))
        {
        	String msg = null;
        	throw new IllegalArgumentException(msg);
        }
        
    } 
}
