/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.part;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.Icon;

import com.onaro.client.leekui.ui.IWorkbenchPart;
import com.onaro.client.leekui.ui.IWorkbenchPartSite;
import com.onaro.client.leekui.ui.internal.PartSite;
import com.onaro.client.leekui.ui.internal.Util;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.part.WorkbenchPart
 */
public abstract class WorkbenchPart implements IWorkbenchPart {
	private PropertyChangeSupport propertyChangeSupport;

	private String contentDescription = ""; //$NON-NLS-1$

	private String partName = ""; //$NON-NLS-1$

	private String title = ""; //$NON-NLS-1$

	private Icon titleIcon;

	private String titleToolTip = ""; //$NON-NLS-1$

	private PartSite partSite;

	public WorkbenchPart() {
		// do nothing
	}

	private PropertyChangeSupport getPropertyChangeSupport() {
		if (propertyChangeSupport == null) {
			propertyChangeSupport = new PropertyChangeSupport(this);
		}

		return propertyChangeSupport;
	}

	protected final void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
		getPropertyChangeSupport().firePropertyChange(propertyName, oldValue, newValue);
	}

	public void dispose() {
		propertyChangeSupport = null;
	}

	public String getTitle() {
		return title;
	}

	public Icon getTitleIcon() {
		return titleIcon;
	}

	public String getTitleToolTip() {
		return titleToolTip;
	}

	public String getContentDescription() {
		return contentDescription;
	}

	public String getPartName() {
		return partName;
	}

	public final void addPropertyChangeListener(PropertyChangeListener pcListener) {
		getPropertyChangeSupport().addPropertyChangeListener(pcListener);
	}

	public final void removePropertyChangeListener(PropertyChangeListener pcListener) {
		getPropertyChangeSupport().removePropertyChangeListener(pcListener);
	}

	public IWorkbenchPartSite getSite() {
		return partSite;
	}

	protected void setSite(IWorkbenchPartSite site) {
		checkSite(site);
		this.partSite = (PartSite) site;
	}

	protected void checkSite(IWorkbenchPartSite site) {
		// do nothing
	}

	protected void setPartName(String partName) {
		String oldPartName = partName;

		this.partName = partName;

		this.firePropertyChange("partName", partName, oldPartName); //$NON-NLS-1$

		setDefaultTitle();
	}

	void setDefaultTitle() {
		String name = getPartName();

		String newTitle = name;

		setTitle(newTitle);
	}

	private void setTitle(String title) {
		String oldTitle = this.title;

		this.title = title;

		this.firePropertyChange("title", oldTitle, title); //$NON-NLS-1$
	}

	/**
	 * {@inheritDoc}
	 * 
	 * Subclasses may override this method (however, if they do so, they should invoke the method on their superclass to
	 * ensure that the Platform's adapter manager is consulted).
	 */
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	public Object getAdapter(Class adapter) {

		/**
		 * This implementation of the method declared by <code>IAdaptable</code> passes the request along to the
		 * platform's adapter manager; roughly <code>Platform.getAdapterManager().getAdapter(this, adapter)</code>.
		 */

		//TODO: re-enable this line in 4.0
		// return Platform.getAdapterManager().getAdapter(this, adapter);
		return null;
	}

	/**
	 * Sets or clears the title icon of this part.
	 * 
	 * @param titleIcon
	 *            the title icon, or <code>null</code> to clear
	 */
	protected void setTitleIcon(Icon titleIcon) 
	{
		Icon oldTitleIcon = this.titleIcon;

		this.titleIcon = titleIcon;
		
		firePropertyChange("titleIcon", oldTitleIcon, titleIcon); //$NON-NLS-1$
	}

	/**
	 * Sets or clears the title tool tip text of this part. Clients should call this method instead of overriding
	 * <code>getTitleToolTip</code>
	 * 
	 * @param toolTip
	 *            the new tool tip text, or <code>null</code> to clear
	 */
	protected void setTitleToolTip(String toolTip) {
		toolTip = Util.safeString(toolTip);

		String oldToolTip = this.titleToolTip;

		this.titleToolTip = toolTip;

		firePropertyChange("titleToolTip", oldToolTip, titleToolTip); //$NON-NLS-1$
	}
}
