/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.plugin;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.ImageIcon;

import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.osgi.framework.Bundle;

import com.onaro.client.leekui.ui.internal.util.BundleUtility;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.plugin.AbstractUIPlugin
 */
public abstract class AbstractUIPlugin extends Plugin {
	/**
	 * Creates and returns a new icon for an image file located within the specified plug-in.
	 * <p>
	 * This is a convenience method that simply locates the image file in within the plug-in (no image registries are
	 * involved). The path is relative to the root of the plug-in, and takes into account files coming from plug-in
	 * fragments. The path may include $arg$ elements. However, the path must not have a leading "." or path separator.
	 * Clients should use a path like "icons/mysample.gif" rather than "./icons/mysample.gif" or "/icons/mysample.gif".
	 * </p>
	 * 
	 * @param pluginId
	 *            the id of the plug-in containing the image file; <code>null</code> is returned if the plug-in does
	 *            not exist
	 * @param imageFilePath
	 *            the relative path of the image file, relative to the root of the plug-in; the path must be legal
	 * @return an image descriptor, or <code>null</code> if no image could be found
	 * @since 3.0
	 */
	public static ImageIcon getIconFromPlugin(String pluginId, String imageFilePath) {
		URL fullPath = getURLFromPlugin(pluginId, imageFilePath);
		if (fullPath == null) return null;
		
		return new ImageIcon(fullPath);
	}
	
	public static String getStringFromPlugin(String pluginId, String textFilePath) {
		URL fullPath = getURLFromPlugin(pluginId, textFilePath);
		if (fullPath == null) return null;
		
		StringBuilder sb = new StringBuilder();
		try {
			InputStream iStream = fullPath.openStream();
			
			InputStreamReader reader = new InputStreamReader(iStream);

			char[] buffer = new char[1024];

			int numRead = reader.read(buffer);

			while (numRead >= 0) {
				sb.append(buffer, 0, numRead);

				numRead = reader.read(buffer);
			}
			iStream.close();
		}
		catch (IOException e) {
			return null;
		}
		
		return sb.toString();
	}
	
	public static URL getURLFromPlugin(String pluginId, String filePath) {
		if (pluginId == null || filePath == null) {
			throw new IllegalArgumentException();
		}

		// if the bundle is not ready then there is no path
		Bundle bundle = Platform.getBundle(pluginId);
		if (!BundleUtility.isReady(bundle)) {
			return null;
		}

		// look for the path (this will check both the plugin and fragment folders
		URL fullPath = BundleUtility.find(bundle, filePath);
		if (fullPath == null) {
			try {
				fullPath = new URL(filePath);
			} catch (MalformedURLException e) {
				return null;
			}
		}
		
		return fullPath;
	}
}
