/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.views;

import java.util.List;

import org.eclipse.core.runtime.CoreException;

import com.onaro.client.leekui.ui.IViewPart;
import com.onaro.client.leekui.ui.IWorkbenchPartDescriptor;

/**
 * This is a view descriptor. It provides a "description" of a given
 * given view so that the view can later be constructed.
 * <p>
 * The view registry provides facilities to map from an extension
 * to a IViewDescriptor.
 * </p>
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * 
 * @see IViewRegistry
 * 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.views.IViewDescriptor
 */
public interface IViewDescriptor extends IWorkbenchPartDescriptor
{
	/**
     * Creates an instance of the view defined in the descriptor.
     * 
     * @return the view part
     * @throws CoreException thrown if there is a problem creating the part
     */
	public IViewPart createView() throws Exception;
	
	/**
     * Returns whether this view allows multiple instances.
     */
	public boolean getAllowMultiple();
	
	 /**
     * Returns the description of this view.
     */
	public String getDescription();
	
	/**
     * Returns an array of strings that represent
     * view's category path. This array will be used
     * for hierarchical presentation of the
     * view in places like submenus.
     * @return list of category tokens or null if not specified.
     */
    public List<String> getCategoryPath();
}
